/** \file boot_common.c
 * boot_common.c
 *
 * This file contains all of the VP-API-II declarations that are Voice
 * Termination Device (VTD) family independent. The implementation in this file
 * is applicable to VCP and VPP classes of devices.
 *
 * Copyright (c) 2008, Zarlink Semiconductor Inc.
 * All rights reserved
 *
 * This software is the property of Zarlink Semiconductor , Inc. Please refer to the
 * Non Disclosure Agreement (NDA) that you have signed for more information
 * on legal obligations in using, modifying or distributing this file.
 */

/* INCLUDES */
#include "vp_api.h"
#include "vp_debug.h"

/* Compile Further only if required */
#if defined (VP_CC_VCP_SERIES) || defined (VP_CC_VPP_SERIES) || defined (VP_CC_VCP2_SERIES)

#include "boot_common.h"

#ifndef VP_NO_COMPRESS
#include "zlib.h"
#ifdef VP_COMPRESS_TEST
#ifndef debug_printf
/* Define the following as necessary to redirect the debug output */
#define debug_printf printf
#endif /* debug_printf */
bool veCompressTestEnable = FALSE;
# endif /* VP_COMPRESS_TEST */
#else /* VP_NO_COMPRESS */
# undef VP_COMPRESS_TEST
#endif /* VP_NO_COMPRESS */


#if defined (VP_CC_VCP_SERIES) || defined (VP_CC_VPP_SERIES)
/**
 * VpVcpVppBootLoad()
 *  Loads an executable image into the VTD (VCP/VPP).
 *
 * Refer to the VoicePath API User's Guide for details about this function.
 */
VpStatusType
VpVcpVppBootLoad(
    VpDevCtxType *pDevCtx,
    VpBootStateType state,
    VpImagePtrType pImageBuffer,
    uint32 bufferSize,
    VpScratchMemType *pScratchMem,
    VpBootModeType validation
    )
{
    void *pDevObj = pDevCtx->pDevObj;
    VpDeviceIdType deviceId;
    uint16p pRecCnt;

    switch(pDevCtx->deviceType) {
#if defined (VP_CC_VCP_SERIES)
        case VP_DEV_VCP_SERIES:
            deviceId = ((VpVcpDeviceObjectType *)pDevObj)->deviceId;
            pRecCnt = &((VpVcpDeviceObjectType *)pDevObj)->recCnt;
            break;
#endif
#if defined (VP_CC_VPP_SERIES)
        case VP_DEV_VPP_SERIES:
            deviceId = ((VpVppDeviceObjectType *)pDevObj)->deviceId;
            pRecCnt = &((VpVppDeviceObjectType *)pDevObj)->recCnt;
            break;
#endif
        default:
            return VP_STATUS_INVALID_ARG;
    }

    if (pImageBuffer == VP_NULL) {
        /* Flash booting not supported for these VTDs. */
        return VP_STATUS_INVALID_ARG;
    }

    /* Pass VCP/VPP specific halt, reset, and validate functions to
     * VpBootLoadInternal(). */
    return VpBootLoadInternal(state, pImageBuffer, bufferSize, pScratchMem,
        validation, deviceId, pRecCnt, VpHbiVcpVppClearCodeMem, VpHbiVcpVppHalt,
        VpHbiVcpVppValidate);
}
#endif /* VP_CC_VCP_SERIES || VP_CC_VPP_SERIES */

VpStatusType
VpBootLoadInternal(
    VpBootStateType state,
    VpImagePtrType pImageBuffer,
    uint32 bufferSize,
    VpScratchMemType *pScratchMem,
    VpBootModeType validation,
    VpDeviceIdType deviceId,
    uint16p pRecCnt,
    VpBootSupportFuncPtrType resetFunc,
    VpBootSupportFuncPtrType haltFunc,
    VpBootSupportFuncPtrType validateFunc)
{
    int newblocks;					/* number of uncompressed blocks available */
    VpImagePtrType pHbiBlock;       /* points to data that is ready to be transmitted */
    bool firstblock;                /* whether we're processing the first
                                       block in the image */
#ifndef VP_NO_COMPRESS
    VpZlibStreamType *stream = &(pScratchMem->stream);  /* data structure for inflate() I/O */
    uint8p pInflated = pScratchMem->inflated;           /* buffer for holding inflated data */
    int retval = Z_OK;
#endif
    
    VP_DOUT(DBG_INFO,("VpBootLoadInternal(): Image length: %ld", bufferSize));

    switch (state) {
        case VP_BOOT_STATE_FIRSTLAST:
        case VP_BOOT_STATE_FIRST:
            firstblock = TRUE;
            break;
        case VP_BOOT_STATE_CONTINUE:
        case VP_BOOT_STATE_LAST:
            firstblock = FALSE;
            break;
        default:
            return VP_STATUS_INVALID_ARG;
    }

    /* User passes scratchMemPtr != VP_NULL for compressed images. */
    if (pScratchMem != VP_NULL) {

#ifdef VP_NO_COMPRESS
        /* If compression isn't compiled in, return with error. */
        return VP_STATUS_INVALID_ARG;
#else
        pHbiBlock = (VpImagePtrType)pInflated;

        if ((state == VP_BOOT_STATE_FIRSTLAST) ||
            (state == VP_BOOT_STATE_FIRST)) {
            /* Initialize data structures for VpZlibInflate().  Ignore return
               value (it's hard-coded to return Z_OK). */
            VpZlibInflateInit2(stream, -1 * VP_COMPRESS_WINDOW_BITS,
                    &pScratchMem->state, pScratchMem->window);

            /* stream->next_in points to the input (deflated) buffer for
             *      inflate().
             * stream->avail_in indicates the number of bytes available there.
             * stream->next_out points to the output (inflated) buffer, which is
             *     filled in by inflate() and must be dumped to VpHbiBootWr().
             * stream->avail_out indicates the amount of space remaining
             *   there. */
            stream->next_out = pInflated;
            stream->avail_out = VP_COMPRESS_OUTBUF_PAGES * 128;

#ifdef VP_COMPRESS_TEST
            if (veCompressTestEnable) {
                stream->adler = adler32(0L, Z_NULL, 0);
            }
#endif

        }
        stream->avail_in = bufferSize;
        stream->next_in = pImageBuffer;
#endif /* !VP_NO_COMPRESS */

    } else {

        /* Uncompressed image.  Make sure length is a multiple of 128 bytes. */
        if ((bufferSize % 128) > 0) {
            VP_DOUT(DBG_ERROR,("VpBootLoadInternal(): Uncompressed length not a multiple of 128: %ld", bufferSize));
            return VP_STATUS_ERR_IMAGE;
        }
    }

    /* As each chunk of pImageBuffer is processed, the length of the chunk will
       be subtracted from bufferSize.  Keep going until bufferSize = 0.  */
#ifdef VP_NO_COMPRESS
    while (bufferSize > 0) {
#else
    while ((bufferSize > 0) && (retval != Z_STREAM_END)) {
        /* If pImageBuffer is compressed... */
        if (pScratchMem != VP_NULL) {

            /* Inflate from pImageBuffer to the pInflated[] buffer. */
            retval = VpZlibInflate(stream, Z_SYNC_FLUSH);

            /* Handle inflate() error conditions. */
            if ((retval != Z_OK) && (retval != Z_STREAM_END)) {

#ifdef VP_COMPRESS_TEST
                if (veCompressTestEnable) {
                    debug_printf("inflate() returned %d\n", retval);
                }
#endif

                haltFunc(deviceId); /* halt the VTD */
                VP_DOUT(DBG_ERROR,("VpBootLoadInternal(): inflate() error %d", retval));
                return VP_STATUS_ERR_IMAGE;
            }

            if (stream->avail_out % 128) {
                /* We ran out of input, but didn't produce an integer number
                   of pages of output (i.e. a multiple of 128 bytes). */
                if (
                    (state == VP_BOOT_STATE_LAST) ||
                    (state == VP_BOOT_STATE_FIRSTLAST)
                ) {

#ifdef VP_COMPRESS_TEST
                    if (veCompressTestEnable) {
                        debug_printf("inflate() = %d; stream->avail_in = %lu;"
                                     " stream->avail_out = %lu\n", retval,
                                     (unsigned long)stream->avail_in,
                                     (unsigned long)stream->avail_out);
                    }
#endif

                    haltFunc(deviceId); /* halt the VTD */
                    VP_DOUT(DBG_ERROR,("VpBootLoadInternal(): Output not a multiple of 128"));
                    return VP_STATUS_ERR_IMAGE;
                } else {
                    /* Expecting another call to VpBootload() to continue. */
                    return VP_STATUS_SUCCESS;
                }
            }

            /* Prepare to process the newly inflated block(s). */
            newblocks = ((VP_COMPRESS_OUTBUF_PAGES * 128) -
                            stream->avail_out) / 128;
            pHbiBlock = (VpImagePtrType)pInflated;

            /* Reset output pointer to the beginning of the output buffer. */
            stream->next_out = pInflated;
            stream->avail_out = VP_COMPRESS_OUTBUF_PAGES * 128;

            /* Reduce bufferSize to match the amount of input data remaining. */
            bufferSize = stream->avail_in;

        } else
#endif /* !VP_NO_COMPRESS */

        {
            /* Prepare to process the blocks directly from pImageBuffer. */
            pHbiBlock = pImageBuffer;
            newblocks = bufferSize / 128;
            bufferSize = 0;
        }

        /* Process the new block(s). */
        while (newblocks--) {

            /* First block in the application image contains special data. */
            if (firstblock) {

#ifdef VP_COMPRESS_TEST
            if (!veCompressTestEnable) {
#endif

                /* Sanity check for magic numbers at the beginning of the
                 * boot stream. */
                if ((pHbiBlock[0] != 0xFE) || (pHbiBlock[1] != 0xFF)) {
                    VP_DOUT(DBG_ERROR,("VpBootLoadInternal(): FEFF not found at beginning"));
                    return VP_STATUS_ERR_IMAGE;
                }

                /* Extract the record count from the boot stream. */
                *pRecCnt = ((uint16)pHbiBlock[6] << 8) + pHbiBlock[7];

                /* Initialize the HBI and reset the VTD. */
                if (
                    (VpHalHbiInit(deviceId) == FALSE) ||
                    (resetFunc(deviceId) == FALSE) /* initiate HBI boot sequence */
                ) {
                    VP_DOUT(DBG_ERROR,("VpBootLoadInternal(): Failed VpHalHbiInit or resetFunc"));
                    return VP_STATUS_ERR_HBI;
                }


#ifdef VP_COMPRESS_TEST
            }
#endif

                firstblock = FALSE;
            }

#ifdef VP_COMPRESS_TEST
        if (veCompressTestEnable) {
            stream->adler = adler32(stream->adler, pHbiBlock, 128);
        } else {
#endif

            /* Transmit the block over the HBI bus. */
            if (!VpHalHbiBootWr(deviceId, 63, pHbiBlock)) {
                haltFunc(deviceId); /* halt the VTD */
                VP_DOUT(DBG_ERROR,("VpBootLoadInternal(): Failed VpHalHbiBootWr"));
                return VP_STATUS_ERR_HBI;
            }

#ifdef VP_COMPRESS_TEST
        }
#endif

            /* Decrement block counter. */
            (*pRecCnt)--;

            /* Increment HBI data pointer to next block. */
            pHbiBlock = &pHbiBlock[128];

        } /* while (newblocks) */
    } /* until we run out of input data */

#ifdef VP_COMPRESS_TEST
if (veCompressTestEnable) {
    if ((state == VP_BOOT_STATE_FIRSTLAST) || (state == VP_BOOT_STATE_LAST)) {
        term_printf("(adler32 = %8.8lXh) ", stream->adler);
    }
} else {
#endif

    /* If we've sent all the blocks, check the hardware checksum and record count. */
    if ((state == VP_BOOT_STATE_FIRSTLAST) || (state == VP_BOOT_STATE_LAST)) {
        if (validation == VP_BOOT_MODE_VERIFY) {

            /* Record count should be 0 after decrementing once for each block sent. */
            if (*pRecCnt != 0) {
                /* debug_printf("recCnt = %lu\n", (unsigned long) recCnt); */
                haltFunc(deviceId);
                VP_DOUT(DBG_ERROR,("VpBootLoadInternal(): Record count not 0 at end"));
                return VP_STATUS_ERR_IMAGE;
            }

            /* Verify HBI load checksum equals expected value. */
            if (validateFunc(deviceId) == FALSE) {
                haltFunc(deviceId);
                return VP_STATUS_ERR_VERIFY;
            }
        }
    }

#ifdef VP_COMPRESS_TEST
}
#endif

    return VP_STATUS_SUCCESS;
} /* VpBootLoadInternal() */

#endif /* VP_CC_VCP_SERIES || VP_CC_VPP_SERIES || VP_CC_VCP2_SERIES */








