/** \file vcp2_api.h
 * vcp2_api.h
 *
 * This header file contains all the declarations and defintions required by
 * an application (in addition to the generic VP-API functions and types) to
 * initialize a VCP2 device.
 *
 * Copyright (c) 2007, Zarlink Semiconductor Inc.
 * All rights reserved
 *
 * This software is the property of Zarlink Semiconductor , Inc. Please refer to the
 * Non Disclosure Agreement (NDA) that you have signed for more information
 * on legal obligations in using, modifying or distributing this file.
 */

#ifndef VCP2_API_H
#define VCP2_API_H

#include "vp_api_types.h"
#include "vp_api_common.h"
#include "vp_api_cfg.h"
#include "vp_api_event.h"

/* The following macros can be used by the application during memory
 * allocation. */

/* Total number of line objects that need to be created for this device: */
#define VP_VCP2_MAX_NUM_LINES (VP_VCP2_MAX_NUM_SLACS * VP_VCP2_MAX_NUM_CHANS)
#if (VP_MAX_LINES_PER_DEVICE < VP_VCP2_MAX_NUM_LINES)
#undef VP_VCP2_MAX_NUM_LINES
#define VP_VCP2_MAX_NUM_LINES VP_MAX_LINES_PER_DEVICE
#endif
#if (VP_VCP2_MAX_NUM_LINES % 2)
#error "VCP2 driver requirement: VP_VCP2_MAX_NUM_LINES must be an even number."
#endif

/* Total number of words of memory that need to be allocated for test buffers
 * when using VpFillTestBuf() with this device: */
#define VP_VCP2_TEST_BUF_MAX_SAMPLES ((4 * 1024 + 128)

/* Number of types of profiles supported by VCP2: */
#define VP_VCP2_NUM_PROFILE_TYPES (VP_PROFILE_TONECAD + 1)


/* The application might use the following macros in calling VpSetOption(): */
/* Events which are not ever generated by the VCP2: */
#define VCP2_INVALID_FAULT_EVENTS           (VP_EVCAT_FAULT_UNDEFINED)
#define VCP2_INVALID_SIGNALING_EVENTS       (VP_EVCAT_SIGNALING_UNDEFINED \
    | VP_LINE_EVID_US_TONE_DETECT | VP_LINE_EVID_DS_TONE_DETECT \
    | VP_DEV_EVID_SEQUENCER)
#define VCP2_INVALID_RESPONSE_EVENTS        (VP_EVCAT_RESPONSE_UNDEFINED)
#define VCP2_INVALID_TEST_EVENTS            (VP_EVCAT_TEST_UNDEFINED \
    | VP_LINE_EVID_TEST_RSVD1)
#define VCP2_INVALID_PROCESS_EVENTS         (VP_EVCAT_PROCESS_UNDEFINED)
#define VCP2_INVALID_FXO_EVENTS             0xFFFF
#define VCP2_INVALID_PACKET_EVENTS          0xFFFF


/* The following type definitions are needed by the application when calling
 * VP-API initialization functions. */

/* The following enumeration type is used to identify the VCP product type.
 * This is must match the product type of the application image downloaded
 * with VpBootLoad(). */
typedef enum {
    /* RAM Product Code Format:
     *     +-----+-----+-----+-----+-----+-----+-----+-----+
     *     |  7  |  6  |  5  |  4  |  3  |  2  |  1  |  0  |
     *     +-----+-----+-----+-----+-----+-----+-----+-----+
     *     |   SLAC    |   Device  |      Feature Set      |
     *     +-----+-----+-----+-----+-----+-----+-----+-----+
     */

    /* Bitfields */
    VP_VCP_PROD_SLAC_MASK       = 0xC0, /* Bitfield distinguishing 790/880/NG */
    VP_VCP_PROD_VTD_MASK        = 0x30, /* Bitfield distinguishing VCP1/VCP2 */
    VP_VCP_PROD_FEAT_MASK       = 0x0F, /* Bitfield distinguishing NT/AT/ATP/Demo */

    /* VTD types */
    VP_VCP_PROD_VTD_VCP1        = 0x00,
    VP_VCP_PROD_VTD_VCP2        = 0x10,

    /* SLAC types */
    VP_VCP_PROD_SLAC_VE790      = 0x00, /* qislac */
    VP_VCP_PROD_SLAC_VE880      = 0x80, /* voiceport 880 */
    VP_VCP_PROD_SLAC_NG         = 0xC0,

    /* Feature sets */
    VP_VCP_PROD_FEAT_NT         = 0x05, /* no test */
    VP_VCP_PROD_FEAT_BT         = 0x02, /* basic test */
    VP_VCP_PROD_FEAT_AT         = 0x04, /* advanced test */
    VP_VCP_PROD_FEAT_DEMO       = 0x06, /* small-linecount demo */
    VP_VCP_PROD_FEAT_NA         = 0x00, /* not applicable */
    VP_VCP_PROD_FEAT_RSVD1      = 0x01, /* obsolete product */
    VP_VCP_PROD_FEAT_RSVD2      = 0x03, /* obsolete product */

    /* VCP1-790 product codes */
    VP_VCP_PROD_VCP1_790_NT =
        (VP_VCP_PROD_VTD_VCP1 | VP_VCP_PROD_SLAC_VE790 | VP_VCP_PROD_FEAT_NT),
    VP_VCP_PROD_VCP1_790_BT =
        (VP_VCP_PROD_VTD_VCP1 | VP_VCP_PROD_SLAC_VE790 | VP_VCP_PROD_FEAT_BT),
    VP_VCP_PROD_VCP1_790_AT =
        (VP_VCP_PROD_VTD_VCP1 | VP_VCP_PROD_SLAC_VE790 | VP_VCP_PROD_FEAT_AT),

    /* VCP1-880 product codes */
    VP_VCP_PROD_VCP1_880_NT =
        (VP_VCP_PROD_VTD_VCP1 | VP_VCP_PROD_SLAC_VE880 | VP_VCP_PROD_FEAT_NA),

    /* VCP2-790 product codes */
    VP_VCP_PROD_VCP2_790_NT =
        (VP_VCP_PROD_VTD_VCP2 | VP_VCP_PROD_SLAC_VE790 | VP_VCP_PROD_FEAT_NT),
    VP_VCP_PROD_VCP2_790_BT =
        (VP_VCP_PROD_VTD_VCP2 | VP_VCP_PROD_SLAC_VE790 | VP_VCP_PROD_FEAT_BT),
    VP_VCP_PROD_VCP2_790_AT =
        (VP_VCP_PROD_VTD_VCP2 | VP_VCP_PROD_SLAC_VE790 | VP_VCP_PROD_FEAT_AT),
    VP_VCP_PROD_VCP2_790_DEMO =
        (VP_VCP_PROD_VTD_VCP2 | VP_VCP_PROD_SLAC_VE790 | VP_VCP_PROD_FEAT_DEMO),

    /* Startup value */
    VP_VCP_PROD_UNKNOWN         = 0x00,

    VP_VCP_PROD_ENUM_SIZE = FORCE_STANDARD_C_ENUM_SIZE
} VpVcp2ProductType;

/* VCP2 device object definition: */
typedef struct {

    /* Customer-defined device identifier struct, passed to HAL functions: */
    VpDeviceIdType deviceId;

    /* Blocks remaining to be downloaded by VpBootLoad(): */
    uint16 recCnt;

    /* Firmware product type (see above): */
    VpVcp2ProductType product;

    /* Total number of channels controlled by this device: */
    uint8 numChannels;

    /* A flag indicating whether VpInitDevice() has completed: */
    bool devInit;

    /* Number of SLACs detected during VpInitDevice(): */
    uint16 slacsFound;

    /* Flags indicating whether profile table slots have been initialized: */
    uint16 profTabInit[VP_VCP2_NUM_PROFILE_TYPES];

    /* Variables for maintaining the high-order bits of the device timestamp: */
    uint16 timeStampHiBits;
    bool timeStampEventMasked;

} VpVcp2DeviceObjectType;

/* VCP2 line object definition: */
typedef struct {

    /* Number used by the VCP2 to uniquely identify this line: */
    uint8 channelId;

    /* Line termination type: */
    VpTermType termType;

    /* Customer-defined line identifier struct, returned in the event
       structure by VpGetEvent() when an event occurs on this line: */
    VpLineIdType lineId;
    
    /* The API internally masks hook events when the line is in the tip-open
       state, to hide a VCP2 firmware erratum. */
    bool tipOpenState;

} VpVcp2LineObjectType;


/* The following enumeration can be used to parse the eventData field in the
   VpEventType struct for a VP_DEV_EVID_SYSTEM_FLT event: */
typedef enum {
    VP_SYS_FLT_VCP2_EVQ_OFL = 0x4000, /* Event queue overflow */
    VP_SYS_FLT_VCP2_WDOG    = 0x2000, /* Watchdog timer expired */
    VP_SYS_FLT_VCP2_CFAIL   = 0x1000, /* VCP2 core clock failure */
    VP_SYS_FLT_VCP2_RSVD    = 0x8FFF, /* reserved bits */
} VpVcp2SysFltEventDataType;

#endif /* VCP2_API_H */
