/** \file vp890_api.h
 * vp890_api.h
 *
 *  Header file that define all the commands for the Vp890 device.
 *
 * Copyright (c) 2008, Zarlink Semiconductor Inc.
 * All rights reserved
 *
 * This software is the property of Zarlink Semiconductor , Inc. Please refer to the
 * Non Disclosure Agreement (NDA) that you have signed for more information
 * on legal obligations in using, modifying or distributing this file.
 */

#ifndef VP792_API_H
#define VP792_API_H

#define VP792_MIN_SLAC_ID                       0
#define VP792_MAX_SLAC_ID                       7
#define VP792_MAX_NUM_CHANNELS                  8

#define VP792_MAILBOX_SPINWAIT               10000

#define VP_792_DEV_PROF_TABLE_SIZE              1
#define VP_792_AC_PROF_TABLE_SIZE               2
#define VP_792_DC_PROF_TABLE_SIZE               2
#define VP_792_RINGING_PROF_TABLE_SIZE          2
#define VP_792_TONE_CADENCE_PROF_TABLE_SIZE     11
#define VP_792_TONE_PROF_TABLE_SIZE             10
#define VP_792_RING_CADENCE_PROF_TABLE_SIZE     4
#define VP_792_METERING_PROF_TABLE_SIZE         2
#define VP_792_CALLERID_PROF_TABLE_SIZE         2
#define VP_792_FXO_CONFIG_PROF_TABLE_SIZE       1
#define VP_792_CUSTOM_TERM_PROF_TABLE_SIZE      4

/* Maximum lengths for various types of profiles (bytes): */
#define VP792_DEV_PROFILE_LEN               38  
#define VP792_DC_PROFILE_LEN                43 
#define VP792_AC_PROFILE_LEN                94 
#define VP792_CID_PROFILE_LEN               88 
#define VP792_MTR_PROFILE_LEN               100 /* to do: determine this precisely */
#define VP792_TONE_PROFILE_LEN              31
#define VP792_RINGCAD_PROFILE_LEN           72
#define VP792_TONECAD_PROFILE_LEN           72
#define VP792_RING_PROFILE_LEN              31

/* Lengths of profile sections that need to be stored or otherwise handled
 * separately from the whole. (words) */
#define VP792_AC_PROFILE_REGDATA_LEN        1
#define VP792_DC_PROFILE_REGDATA_LEN        17
#define VP792_RING_PROFILE_REGDATA_LEN      27

#include "vp_hal.h"
#include "vp_CSLAC_types.h"


typedef struct {
    VpProfilePtrType pDevProfileTable[VP_792_DEV_PROF_TABLE_SIZE];
    VpProfilePtrType pAcProfileTable[VP_792_AC_PROF_TABLE_SIZE];
    VpProfilePtrType pDcProfileTable[VP_792_DC_PROF_TABLE_SIZE];
    VpProfilePtrType pRingingProfileTable[VP_792_RINGING_PROF_TABLE_SIZE];
    VpProfilePtrType pToneCadProfileTable[VP_792_TONE_CADENCE_PROF_TABLE_SIZE];
    VpProfilePtrType pToneProfileTable[VP_792_TONE_PROF_TABLE_SIZE];
    VpProfilePtrType pRingingCadProfileTable[VP_792_RING_CADENCE_PROF_TABLE_SIZE];
    VpProfilePtrType pMeteringProfileTable[VP_792_METERING_PROF_TABLE_SIZE];
    VpProfilePtrType pCallerIdProfileTable[VP_792_CALLERID_PROF_TABLE_SIZE];
    VpProfilePtrType pFxoConfigProfileTable[VP_792_FXO_CONFIG_PROF_TABLE_SIZE];
    VpProfilePtrType pCustomTermProfileTable[VP_792_CUSTOM_TERM_PROF_TABLE_SIZE];
} Vp792DeviceProfileTableType;

typedef union {
    VpOptionCriticalFltType criticalFaultOption;
    VpOptionEventMaskType eventMaskOption;
    VpOptionPulseModeType pulseModeOption;
    VpOptionPulseType pulseOption;
    VpOptionRingControlType ringControlOption;
    VpOptionZeroCrossType zeroCrossOption;
    VpOptionTimeslotType timeSlotOption;
    VpOptionCodecType codecOption;
    VpOptionPcmHwyType pcmHwyOption;
    VpOptionLoopbackType loopBackOption;
    VpOptionLineStateType lineStateOption;
    VpOptionPcmTxRxCntrlType pcmTxRxCtrlOption;
    VpOptionDtmfSpecType dtmfSpecOption;
    VpOptionLineIoConfigType lineIoCfgOption;
} Vp792GetOptionResultsDataType;

typedef struct {
    Vp792GetOptionResultsDataType optionData;
    uint8 chanId;
    VpOptionIdType optionType;
} Vp792GetOptionResultsType;

/* The following enumeration type is used to identify the 792 product type.
 * This is must match the product type of the DEVCODE register */
typedef enum {
    /* Startup value */
    VP_792_PROD_UNKNOWN         = 0x00,

    VP_792_PROD_ENUM_SIZE = FORCE_STANDARD_C_ENUM_SIZE
} Vp792ProductType;

/* Vp792 specific Device Object */
typedef struct {
    /* Customer-defined device identifier struct, passed to HAL functions: */
    VpDeviceIdType  deviceId;

    /* VP-API slac identifier type, returned in the event
     * structure by VpGetEvent() when an event occurs on this slac:
     * This member allows for multiple device objects to share a common
     * deviceId */
    uint8           slacId;

    /* Total number of channels controlled by this device: */
    uint8           maxChannels;

    /* Revision Code info */
    uint16          revCode;

    /* PCM Clock Rate in units of 1KHz */
    uint16          pcmClockRate;

    /* A flag indicating whether VpInitDevice() has completed: */
    bool            devInit;

    /* Flags indicating whether profile table slots have been initialized: */
    Vp792DeviceProfileTableType     profileTables;

    /* Firmware product type (see above): */
    bool            extRingSupport;

    /* Software mask for device events */
    VpOptionEventMaskType deviceEventsMask;

    /* Temporary storage for interrupt indication, while mailbox is busy. */
    uint16 intInd[2];

    /* Flag to indicate that GetResults must be called before doing another
     * operation with extended results. */
    bool            resultFlag;

    /* Handle associated with a read request */
    uint16          resultHandle;

    /* Structure to hold results from GetOption */
    Vp792GetOptionResultsType getOptionResults;

    /* Structure to hold results from GetLoopCond */
    VpLoopCondResultsType rdLoopResults;
    
    /* Structure to hold results for a LineIoAccess read */
    VpLineIoAccessType lineIoReadResults;

    /* Bitmask indicating which device-specific event handlers are running. */
    uint16 eventHandlers;

    /* Increments by 16384 with each timestamp rollover event from the device.
     * Add (device timestamp)/4 to get the API timestamp. */
    uint16 timeStamp;

    /* We save the command words for a low level paged read command so that we
     * can perform the actual read when the user calls VpGetResults() instead of
     * storing a potentially large buffer of read data. */
    uint16 lowLevReadCmd[2];  
    uint8 lowLevReadCmdLen;
    
    /* Save the number of words to read and the channelId for low level
     * mailbox reads */
    uint8 lowLevReadChan;
    uint8 lowLevReadWords;
    
    /* When VpGetResults() is called in response to a LLCMD_RX_CMP event, we
     * need to know whether the low level read was a page or mailbox read. */
    VpLowLevelCmdType lowLevReadType;
    
    /* Save the LLC handle for reads so that we know when to treat a MB_RSP
     * event as a low level read response, and so that we can return the same
     * handle in the LLCMD_RX_CMP event. */
    uint16 lowLevReadHandle;

} Vp792DeviceObjectType;

/* Vp792 specific Line Object */
typedef struct {

    /* Number used by the Vp792 to uniquely identify this line: */
    uint8               channelId;

    /* Line termination type: */
    VpTermType          termType;

    /* Customer-defined line identifier struct, returned in the event
     * structure by VpGetEvent() when an event occurs on this line: */
    VpLineIdType        lineId;

    /* Current profiles: */
    struct {
        VpProfilePtrType pRingCadProfile;
        VpProfilePtrType pToneProfile;
        VpProfilePtrType pToneCadProfile;
        VpProfilePtrType pCidProfile;
    } profiles;
    
    struct {
        VpProfileDataType acProfileRegData[VP792_AC_PROFILE_REGDATA_LEN];
        VpProfileDataType dcProfileRegData[VP792_DC_PROFILE_REGDATA_LEN];
        VpProfileDataType ringProfileRegData[VP792_RING_PROFILE_REGDATA_LEN];
    } profilesData;
    
    /* Bitfield to be set in Vp792InitLine and used in EhInitLine indicating
     * which profiles (AC, DC, Ring) */
    uint8 initLineValidProfiles;

    /* Current API line state */
    VpLineStateType     currentState;

    /* State that is automatically switched to when ring-trip occurs */
    VpLineStateType     ringTripExitSt;

    /* Zero Cross Control option value stored by SetOption for GetOption */
    VpOptionZeroCrossType zeroCrossMode;

    /* Software mask for line events */
    VpOptionEventMaskType lineEventsMask;

    /* Defines how the PCM highway is set for "talk" linestates */
    VpOptionPcmTxRxCntrlType pcmTxRxCtrl;

    /* Keeps track of whether the sequencer is currently running. */
    bool sequencerRunning;
    
    /* Determines whether the API reports raw hooks or decoded pulse events */
    bool pulseDecodeOn;

    /* Bitmask indicating which line-specific event handlers are running. */
    uint16 eventHandlers;
    
    /* Stores VpSendSignal data so that signal requests can be completed using
     * the event handlers. */
    VpSendSignalType signalData;
    
    /* Stored handle to associate VpLineIoAccess() calls with 
     * VP_LINE_EVID_LINE_IO_XX_CMP events. */
    uint16 lineIoAccessHandle;
    
    /* Caller ID FSK data buffer */
    struct {
        /* device buffer status */
        uint8 bufStatus;

        /* Preamble */
        uint16 seizureBits;
        uint16 markBits;

        /* Message data */
        uint8 buf[VP_SIZEOF_CID_MSG_BUFFER * 2];
        uint8 head;
        uint8 tail;
        bool nonempty;
        uint8 checksum;
    } callerId;

    /* State variables for EhSequenceAborted() event handler: */
    struct {
        VpLineStateType newState;
    } ehSequenceAborted;
    
} Vp792LineObjectType;

/* Vp792 Group Device Object */

typedef struct {

    VpDeviceIdType deviceId;

    /* Temporary storage for interrupt indication, while mailbox is busy. */
    uint16 intInd[2];

} Vp792GroupDeviceObjectType;

#endif /* VP792_API_H */
