/** \file vp890_api.h
 * vp890_api.h
 *
 *  Header file that define all the commands for the Vp890 device.
 *
 * Copyright (c) 2008, Zarlink Semiconductor Inc.
 * All rights reserved
 *
 * This software is the property of Zarlink Semiconductor , Inc. Please refer to the
 * Non Disclosure Agreement (NDA) that you have signed for more information
 * on legal obligations in using, modifying or distributing this file.
 */


#ifndef VP890_API_H
#define VP890_API_H

#include "vp_hal.h"
#include "vp_CSLAC_types.h"

#ifdef VP890_INCLUDE_TESTLINE_CODE
#include "vp_api_test.h"
  #ifdef VP890_EZ_MPI_PCM_COLLECT
  #include  "vp_pcm_compute.h"
  #endif
#endif

#define VP890_MAX_NUM_CHANNELS   2

#define VP890_MAX_MPI_DATA      15   /* Max data from any MPI read command */
#define VP890_INT_SEQ_LEN       22

#ifndef VP890_UL_SIGREG_LEN
#define VP890_UL_SIGREG_LEN         0x02
#endif

#ifndef VP890_SIGA_PARAMS_LEN
#define VP890_SIGA_PARAMS_LEN       0x0B
#endif

#ifndef VP890_DC_FEED_LEN
#define VP890_DC_FEED_LEN           0x02
#endif


#ifndef VP890_ICR1_LEN
#define VP890_ICR1_LEN              0x04
#endif

#ifndef VP890_ICR2_LEN
#define VP890_ICR2_LEN              0x04
#endif

#ifndef VP890_ICR3_LEN
#define VP890_ICR3_LEN              0x04
#endif

#ifndef VP890_ICR4_LEN
#define VP890_ICR4_LEN              0x04
#endif

#ifndef VP890_DC_CAL_REG_LEN
#define VP890_DC_CAL_REG_LEN              0x02
#endif

#ifndef VP890_LOOP_SUP_LEN
#define VP890_LOOP_SUP_LEN          0x04
#endif

#ifndef VP890_TEST_DATA_LEN
#define VP890_TEST_DATA_LEN         0x0E
#endif

#ifndef VP890_REGULATOR_PARAM_LEN
#define VP890_REGULATOR_PARAM_LEN   0x03
#endif

#ifndef VP890_REGULATOR_TIMING_LEN
#define VP890_REGULATOR_TIMING_LEN   0x06
#endif

#ifndef VP890_REGULATOR_TIMING_LEN
#define VP890_REGULATOR_TIMING_LEN  0x06
#endif

/**< Required Vp890 Device and Line Objects for user instantiation if a Vp890
 * device is used
 */

/**< Structure that defines the Vp890 Device Profile. Current as of the first
 * Device Profile version (ver 0).
 */
typedef struct {
    uint8   maxNumInterrupts;
    uint16  pcmClkRate;      /**< Used to verify valid TX/RX Timeslot setting */
    uint8   mClkMask;
    uint16  tickRate;        /**< Primary API-II tick for this device */
    uint8   devCfg1;
    uint8   clockSlot;
    uint8   switchingParams[3];
} Vp890DeviceProfileType;

#if defined (VP890_INCLUDE_TESTLINE_CODE)
/*
 * #defines used in unit conversion function
 * All units are based on the following calculations
 *      Internal voltage unit   = 480.0  / 2^31 Volts
 *      Internal current unit   = 240e-3 / 2^31 Amps
 *      Internal freq uint      = 12e3   / 2^31 Hz
 */
#define VP890_UNIT_MV                          4474l    /* mV unit */
#define VP890_UNIT_MVRMS                       6327l    /* mVrms unit */
#define VP890_UNIT_ADC_VAB                     -32768l  /* vab step from ADC */
#define VP890_UNIT_ADC_VAB_RMS                 46341l   /* vab step from ADC rms*/
#define VP890_UNIT_DAC_RING                    21076l   /* step due to DAC in ringing*/
#define VP890_UNIT_UA                          8948l    /* uA unit */
#define VP890_UNIT_ADC_IMET_NOR                16384l   /* met I from ADC in normal*/
#define VP890_UNIT_ADC_IMET_RING               32768l   /* met I from ADC in ringing*/
#define VP890_UNIT_ADC_ILG                     11469    /* long current from ADC*/
#define VP890_UNIT_DECHZ                       17896l   /* Deci Hz*/
#define VP890_UNIT_MHZ                         179l     /* mHz */
#define VP890_UNIT_FREQ                        65536l   /* siggen freq step */
#define VP890_UNIT_RAMP_TIME                   1365l    /* Ramp time (-1/3) in ms*/

#define VP890_UNIT_CONVERT(DATA, I_UNIT, O_UNIT) (int32)( ((int32)DATA * I_UNIT) / O_UNIT)

/* converts requested slope into SigGen Freq */
#define VP890_SLOPE_TO_FREQ(AMP,SLOPE) (int16) ( ((int32)SLOPE * (int32)VP890_UNIT_RAMP_TIME) / (int32)AMP)

/* Definitions for Test arguments */
typedef union {
    VpTestConcludeType          conclude;
    VpTestOpenVType             openV;
    VpTestDcRLoopType           dcRloop;
    VpTestAcRLoopType           acRloop;
    VpTest3EleResAltResType     resFltAlt;
    VpTestLoopCondType          loopCond;
    VpTestLoopbackType          loopback;
    VpTestRampType              ramp;
    VpTestRampInitType          rampInit;
} Vp890TestArgsType;

/* Used to configure the device for a target current */
typedef struct {
    bool success;
    int16 polarity;
    int16 state;
    int16 nullI;
    int16 metI;
    uint8 prevSigCtrl;
    uint8 prevSlacState;
    uint8 prevVpGain;
    uint8 prevDcFeed[VP890_DC_FEED_LEN];
} Vp890TargetCurrentType;

typedef struct {
    uint8 adcState;
    int16 nextState; /**< Used when a pcm collection routine is started */

    Vp890TestArgsType testArgs; /**< Saved test input arguments of current test */

    uint8 opCond;                           /**< Saved Operation Condition */
    uint8 opFunction;                       /**< Saved Operation Functions */
    uint8 opMode;                           /**< Saved Operating Mode */
    uint8 sysConfig;                        /** Saved System Configurations */
    uint8 sigCtrl;                          /**< Signal Generator Control */
    uint8 convCfg;                          /**< Saved Converter Config */
    uint8 slacState;                        /**< Saved Slac State */
    uint8 vpGain;                           /**< Voice Path Gain */
    uint8 disn;                             /**< Digital Imped. Scaling Network */
    uint8 lpSuper[VP890_LOOP_SUP_LEN];      /**< Saved Loop Sup. Parameters */
    uint8 dcFeed[VP890_DC_FEED_LEN];        /**< Saved DC Feed Parameters */
    uint8 switchReg[VP890_REGULATOR_PARAM_LEN]; /**< Switching Reg Parameters */
    uint8 switchTimes[VP890_REGULATOR_TIMING_LEN]; /**< Switching Reg Timing */
  
    uint8 icr1[VP890_ICR1_LEN];
    uint8 icr2[VP890_ICR2_LEN];
    uint8 icr3[VP890_ICR3_LEN];
    uint8 icr4[VP890_ICR4_LEN];
    uint8 icr6[VP890_DC_CAL_REG_LEN];
    uint8 sigGenAB [VP890_SIGA_PARAMS_LEN]; /**< Saved Signal Generator A & B*/
    
    /* used for collecting PCM data */
    bool pcmRequest;        /** < indication that pcm data was requested */
    VpPcmOperationMaskType operationMask;

    VpPcmOperationResultsType pcmResults; /** < stores the pcm operation results */

    /* Used for setting up a current measurment */
    Vp890TargetCurrentType targetI;

    /* Used for common setup functions */
    uint16 commonSetupState;

    /* Used for storing line event mask data */
    VpOptionEventMaskType preTestEventMask;

    /* Used for saving and restoring slac state after calibration */
    uint8 calSlacState;

    /* Used for resflt lg speed up*/
    int16 previousAvg;
    uint16 speedupTime;

    /* The following members are for EZ mode calculations only*/
#ifdef VP890_EZ_MPI_PCM_COLLECT
    VpPcmComputeTempType ezPcmTemp;
#endif

} Vp890TestHeapType;

typedef struct {
    Vp890TestHeapType *pTestHeap;
    uint8 testHeapId;

    uint8 channelId;    /**< Channel # for "this" line on the device.  Indexed
                         * starting at 0, should not exceed the max number of
                         * lines supported by the device - 1 (max = 2, then
                         * channelId = {0, 1}
                         */

    bool prepared;    /**< indicates if the current test is prepared */
    bool concluding;    /**< indicates that the device is concluding a test */
    VpTestIdType testId; /** < indicates the test currently running */

    int16 testState;   /**< maintains the currnt state of the current TestId */
    uint16 handle;

} Vp890CurrentTestType;

typedef struct {
    int16 nullOffset;
    int16 vabOffset;
    int16 vahOffset;
    int16 valOffset;
    int16 vbhOffset;
    int16 vblOffset;
    int16 batOffset;
} Vp890CalOffCoeffs;

#endif /*VP890_INCLUDE_TESTLINE_CODE*/

typedef enum {
    VP890_CAL_INIT = 0,
    VP890_CAL_ERROR,
    VP890_CAL_OFFSET,
    VP890_CAL_MEASURE,
    VP890_CAL_CONVERTER_CHANGE,
    VP890_CAL_ADC,
    VP890_CAL_STATE_CHANGE,
    VP890_CAL_INVERT_POL,
    VP890_CONVERTER_CHECK,
    VP890_CAL_DONE,
    VP890_CAL_EXIT
} Vp890CalState;

typedef enum {
    VP890_BFILT_MEASURE,
    VP890_BFILT_END
} Vp890BFiltCalStates;

#ifndef VP890_B1_FILTER_LEN
#define VP890_B1_FILTER_LEN         0x0E
#endif

#ifndef VP890_B2_FILTER_LEN
#define VP890_B2_FILTER_LEN         0x02
#endif

#ifndef VP890_OP_FUNC_LEN
#define VP890_OP_FUNC_LEN           0x01
#endif

#ifndef VP890_OP_COND_LEN
#define VP890_OP_COND_LEN           0x01
#endif

#ifndef VP890_CONV_CFG_LEN
#define VP890_CONV_CFG_LEN          0x01
#endif

#ifndef VP890_DISN_LEN
#define VP890_DISN_LEN              0x01
#endif

typedef struct {
    VpCalBFilterType inputData;
    uint8 b1FiltData[VP890_B1_FILTER_LEN];
    uint8 b2FiltData[VP890_B2_FILTER_LEN];
    uint8 opFunct[VP890_OP_FUNC_LEN];
    uint8 opCond[VP890_OP_COND_LEN];
    uint8 convCfg[VP890_CONV_CFG_LEN];
    uint16 vRms;
    uint8 listLength;   /* In number of B-Filter coefficient sets */
    uint8 currentSet;   /* Number of currently running B-Filter set */
} Vp890CalBFilterData;

#define VP890_BFILT_SAMPLE_TIME     (70)

typedef enum {
    VP890_RESET             = 0x0000,   /**< Set at the begining of init device */
    VP890_IS_SINGLE_CHANNEL = 0x0020,   /**< Set when a single channel device is found*/
    VP890_DEVICE_DETECTED   = 0x0040,   /**< Set when the device is detected */
    VP890_LINE0_IS_FXO      = 0x0080,   /**< Set if device detection indicates line0 as FXO */
    VP890_LINE1_IS_FXO      = 0x0100,   /**< Set if device detection indicates line1 as FXO */
    VP890_WIDEBAND          = 0x0200,   /**< Set if device supports Wideband mode */
    VP890_LINE0_LP          = 0x0400,   /**< Set if line 0 allows low power mode */
    VP890_LINE1_LP          = 0x0800    /**< Set if line 1 allows low power mode */    
} Vp890DeviceStateIntType;


/**< Line Status types to minimize code space in line object (compared to each
 * status being maintined by a uint8 type)
 */
typedef enum {
    VP890_INIT_STATUS   = 0x0000,

    VP890_IS_FXO        = 0x0001,   /**< Set if the line is configured for FXO */

    VP890_BAD_LOOP_SUP  = 0x0002,   /**< Set when the Loop Supervision has been
                                     * changed in such a way inconsistent with
                                     * the user's specifications. This is done
                                     * in internal to the API to make some
                                     * functions work (e.g., Msg Wait Pulse).
                                     */
                                     
    VP890_DP_SET1_DONE  = 0x0004,   /**< Set when Dial Pulse detection machine
                                     * is "done" on the current dial pulse using
                                     * the "first" set of DP parameters
                                     */

    VP890_DP_SET2_DONE  = 0x0008,   /**< Set when Dial Pulse detection machine
                                     * is "done" on the current dial pulse using
                                     * the 2nd set of DP parameters
                                     */
                                     
    VP890_SLS_CALL_FROM_API = 0x0010,   /**< Set if Set Line State is called
                                         * from an API function (e.g., cadence).
                                         */

    VP890_LINE_IN_CAL = 0x0020      /**< Set while line running cal routine */

} Vp890LineStatusType;

typedef struct {
    uint8   mpiLen;   /**< Length of data to be copied into mpiData buffer */

    uint8   mpiData[VP890_MAX_MPI_DATA];
} Vp890MpiBufferType;

typedef struct {
    uint16  volts;
    uint16  gain;
} Vp890CidCorrectionPair;

typedef enum {
    VP890_NOISEFIX_ST_OFF,          /* No adjustment applied; taking samples */
    VP890_NOISEFIX_ST_ON,           /* Adjustment applied; taking samples */
    VP890_NOISEFIX_ST_FREEZE,       /* This state freezes the siggen */
    VP890_NOISEFIX_ST_RESUME,       /* Resume from pause state; sync with tick*/
    VP890_NOISEFIX_ST_OFFPAUSED,    /* Paused for tone; no adjustment */
    VP890_NOISEFIX_ST_ONPAUSED,     /* Paused for tone; need to reapply fix */
    VP890_NOISEFIX_ST_DISABLED,     /* Disabled - do nothing */
    VP890_NOISEFIX_ST_ENUM_SIZE = FORCE_STANDARD_C_ENUM_SIZE/* Portability Req*/
} Vp890NoiseFixStateType;

typedef struct {
    int16 imtPrev;
    uint8 icr2Data[VP890_ICR2_LEN];
    bool secondPass;
} Vp890VasCalData;

typedef struct {
    int16 ilaOffsetNorm;
    int16 ilaNorm;

    uint8 icr2Data[VP890_ICR2_LEN];
} Vp890IlaCalData;

typedef struct {
    int16 vocNorm;
    int16 vocRev;
    int16 vocOffsetNorm;
    int16 vocOffsetRev;

    uint8 icr6Data[VP890_DC_CAL_REG_LEN];
} Vp890VocCalData;

typedef struct {
    int16 swyOffset[1]; /**< One per channel used to measure SWY Offset */
    int16 swyVolt[1];   /**< One per channel used to measure SWY Voltage */

    uint8 swCal;        /**< d_swcal fine tuning register Result */
    
    uint8 isrpData[VP890_REGULATOR_TIMING_LEN];
    uint8 disnVal[VP890_DISN_LEN][1];
} Vp890AbvCalData;

typedef struct {
    Vp890VasCalData vasData;
    Vp890IlaCalData ilaData;
    Vp890VocCalData vocData;
    Vp890AbvCalData abvData;
    Vp890CalBFilterData bFilterData;
} Vp890CalTypeData;

typedef struct {
    bool calDone;           /**< TRUE when calibration has been performed on
                             * this line
                             */
    bool reversePol;        /**< TRUE when line is in reverse polarity */
    
    Vp890CalTypeData typeData;

    uint8 codecReg;
    uint8 opCondReg;

    int16 adcPcm;                               /* Adc PCM red all Cal */
    int16 calSet;                               /* user set for all Cal */

    uint8 dcFeedRef[VP890_DC_FEED_LEN];         /* Copied from dc profile */
    uint8 dcFeed[VP890_DC_FEED_LEN];            /* use for VAS and VOC */
    uint8 dcFeedPr[VP890_DC_FEED_LEN];          /* use for VAS and VOC */

    Vp890CalState calState;
    VpLineStateType lineState;

    int16 imtPrev;
    uint8 icr2Data[VP890_ICR2_LEN];
    uint8 vocSetting;

    uint8 disnVal[VP890_DISN_LEN];
} Vp890CalLineData;

/* Vp890 specific Device Object */
typedef struct {
    VpDeviceIdType                  deviceId;           /* Device Id indication */

    VpDeviceStaticInfoType          staticInfo;         /**< Info that will not change during runtime. */
    VpDeviceDynamicInfoType         dynamicInfo;        /**< Info that will change during runtime */    

    VpCSLACDeviceStatusType         status;
    Vp890DeviceStateIntType         stateInt;
   
    VpOptionEventMaskType           deviceEventsMask;
    VpOptionEventMaskType           deviceEvents;  
    uint16                          eventHandle;        /** Application defined event handle */  
    uint16                          timeStamp;          /**< Used to track event timing. Increment by ticks */      
    VpGetResultsOptionsType         getResultsOption;  
    VpOptionCriticalFltType         criticalFault;
    
    VpRelGainResultsType            relGainResults;
    
    uint16                          devTimer[VP_DEV_TIMER_LAST];

    Vp890DeviceProfileType          devProfileData;
    VpCSLACDeviceProfileTableType   devProfileTable;
    VpCSLACProfileTableEntryType    profEntry;

    VpTestResultType                testResults;

    uint8                           intReg[VP890_UL_SIGREG_LEN];  /**< Holds signaling data info for the device */

    uint8                           responseData;   /**< Holds data for Response events on the device */

    uint8                           txBufferDataRate;   /**< Buffer for MPI Low level reads to hold 
                                                         * maximum amount of MPI data that is possible  */
    Vp890MpiBufferType              mpiBuffer;
    uint8                           mpiData[VP890_MAX_MPI_DATA];  /**< Buffer for MPI Low level reads to
                                                                   * hold maximum amount of MPI data that
                                                                   * is possible
                                                                   */
    uint8                           mpiLen;       /**< Length of data to be copied into mpiData buffer */

    /*  Two sets of dial pulse specifications are provide to support NTT dial
     * pulse detection windows of 10pps and 20pps while excluding 15pps */
    VpOptionPulseType pulseSpecs;
    VpOptionPulseType pulseSpecs2;

    uint8 testDataBuffer[VP890_TEST_DATA_LEN];
    
#if defined (VP890_INCLUDE_TESTLINE_CODE)
    Vp890CurrentTestType currentTest;
    Vp890CalOffCoeffs calOffsets[VP890_MAX_NUM_CHANNELS];
#endif /* VP890_INCLUDE_TESTLINE_CODE */

} Vp890DeviceObjectType;

/* Kernel specific Line Object */
typedef struct {
    uint8                           channelId;
    uint8                           ecVal;

    VpTermType                      termType;

    VpLineIdType                    lineId;             /* Application provided value for mapping a
                                                         * line to a line context */

    VpApiIntLineStateType           lineState;          /**< Line state info used for state
                                                         * transition handling and recall */
    Vp890LineStatusType             status;

    VpOptionCodecType               codec;

    uint8                           nextSlicValue;

    VpCslacTimerStruct              lineTimers;         /**< Timers for "this" line */
    VpOptionEventMaskType           lineEvents;
    VpOptionEventMaskType           lineEventsMask;
    uint16                          lineEventHandle;    /**< Line specific event handle information */
    uint16                          signaling1;
    uint16                          signaling2;

    uint16                          dtmfDigitSense;     /**< Used to hold the DTMF digit reported
                                                         * with VpDtmfDigitDetected() until the
                                                         * VP_LINE_EVID_DTMF_DIG is generated. */
    uint8                           signalingData;      /**< Holds data for Signaling events on this line */
    uint8                           fxoData;            /**< Holds data for FXO events on this line */
    uint8                           processData;        /**< Holds data for Process events on this line */
    uint8                           responseData;       /**< Holds data for Response events on this line */
    
    uint8                           ringFlag;           /**< Indicates ringing amplitude detection */
    
    uint8                           fxoRingStateFlag;   /**< Indicates a state change request during ringing,
                                                             to be serviced when ringing is over */            
    VpLineStateType                 fxoRingState;       /**< State to change to when ringing is over */ 
    
    uint8                           fxoPohFlag;         /**< POH interrupt flag, set in ServiceFxoInterrupts
                                                             and used in ProcessFxoLine */ 
    
    /*
     * Array to hold ringing parameters used in the Signal Generator.  This is
     * needed when signal generator A is set to a tone, then set back to ringing
     * without the user re-specifying the ringing profile
     */
    uint8                           ringingParams[VP890_SIGA_PARAMS_LEN];

    VpProfilePtrType                pRingingCadence;    /**< Currently used ringing cadence on
                                                         * this line */
                                                         
    VpOptionRingControlType         ringCtrl;                                                                                                                      

    VpProfilePtrType                pCidProfileType1;   /**< Currently used caller ID profile
                                                         * on this line for sequenced cid */

    VpProfilePtrType                pCidProfileType2;   /**< Currently used caller ID profile
                                                         * on this line for non-sequenced cid */

    VpDialPulseDetectType           dpStruct;           /**< Used on FXS lines for detecting pulse
                                                         * digits*/

    VpDialPulseDetectType           dpStruct2;          /**< Used on FXS lines for detecting pulse
                                                         * digits using 2nd set of dp specs. */
    
    VpOptionPulseModeType           pulseMode;
                         
    VpDigitGenerationDataType       digitGenStruct;     /**< Used on FXO lines for
                                                         * generating pulse digits */
                                                         
    uint8                           ringDetMax;         /**< Stores the user specified maximum 
                                                         * ringing detect period for FXO lines. This
                                                         * value may be outside the device range, in
                                                         * which case the SW will implement upper 
                                                         * period detection */
                                                         
    uint8                           ringDetMin;         /**< Stores the user specified maximum 
                                                         * ringing detect period for FXO lines that
                                                         * is within the device range. This value is
                                                         * used as "minimum" that is detected by SW.
                                                         * Actual minimum period is supported by the
                                                         * device period detector itself. */
                                                         
    uint8                           dPoh;               /**< Stores the user specified parallel
                                                         * offhook integration time for use
                                                         * in timing for the POH event */

    VpOptionPcmTxRxCntrlType        pcmTxRxCtrl;   /* Defines how the PCM highway is
                                                    * set for "talk" linestates */
   
    VpRelayControlType              relayState;    /* Used to hold current line 
                                                    * relay state */

    /* Holds user definition for Loop Supervision configuration when
     * "badLoopSup" is TRUE */
    uint8                           loopSup[VP890_LOOP_SUP_LEN];

    uint16                          gxBase;       /**< Cached GX register, in 2.14 int format */
    uint16                          gxCidLevel;   /**< GX Caller ID correction factor (2.14) */
    uint16                          gxUserLevel;  /**< User specified relative GX level (2.14) */
    uint16                          grBase;       /**< Cached GR register, in 2.14 int format */
    uint16                          grUserLevel;  /**< User specified relative GR level (2.14) */

    uint8                           userDtg;             /* User-configured DTG setting */
    uint8                           cidDtg;              /* DTG register setting for CID
                                                          * correction */
    int8                            cidCorrectionSample; /* Last line voltage sample for CID
                                                          * correction */
    uint8                           cidCorrectionCtr;    /* Counter for CID correction line 
                                                          * voltage samples */
                                                          
    Vp890NoiseFixStateType          noiseFixState;  /* State for the A-D noise issue fix */
    int32                           noiseFixSum;    /* Sum of samples for noise fix */
    uint8                           noiseFixSamples;/* Count of samples for noise fix */
    uint8                           noiseFixAvgs;   /* Number of consec. averages in range */
    uint8                           noiseFixAdj;    /* Adjustment value to reach target */

    Vp890CalLineData                calLineData;    /* Data used during VpCalLine() */

#ifdef CSLAC_SEQ_EN
    VpCallerIdType                  callerId; /**< Caller ID related information */
    VpSeqDataType                   cadence;  /**< Sequencer related information */
    VpCidSeqDataType                cidSeq;   /**< CID Sequencer related information */

    bool                            suspendCid;

    /**< Array to control internally run sequences */
    VpProfileDataType               intSequence[VP890_INT_SEQ_LEN];
#endif

} Vp890LineObjectType;


#endif /* VP890_API_H */
