/** \file qs_le71hr0823_connect.c
 * qs_le71hr0823_connect.c
 *
 * This application is provided to demonstrate an FXO to FXS connection over the
 * PCM highway.  The FXO is set to Loop Open/Close when a Loop Open/Close is
 * detected on the FXS line.  The FXS line is set to Feed/Ringing when the FXO
 * line detects Feed/Ringing.
 *
 * Copyright (c) 2008, Zarlink Semiconductor Inc.
 * All rights reserved
 *
 * This software is the property of Zarlink Semiconductor , Inc. Please refer to the
 * Non Disclosure Agreement (NDA) that you have signed for more information
 * on legal obligations in using, modifying or distributing this file.
 */

/*
 * These headers are for UVB/Linux purposes only.  The user will not find them
 * in the API
 */
#include <sys/time.h>
#include <signal.h>
#include <stdio.h>
#include <stdlib.h>
#include "uvb_xs.h"

/*
 * These files are required by all applications.  "vp_api.h" includes all of the
 * API libraries required by the application.  "profiles8.h" includes the
 * profiles used by the application.  At a minimum, there must be a Device
 * Profile for the device to initialize.
 */
#include "vp_api.h"
#include "sys_service.h"
#include "profiles8.h"

/* UVB specific values for device ID depending on the MPI Mode */
#define MPI_3WIRE 0x30
#define MPI_4WIRE 0x10

/* Current configuration of Line Module */
#define MPI_MODE MPI_4WIRE

/* Application specific #define to improve code readability */
#define FXS_DEVICE    0     /**< First device on LM */
#define FXO_DEVICE    1     /**< Second device on LM */
#define NUM_LINES     2     /**< Total number of lines on LM */
#define NUM_DEVICES   2     /**< Total number of devices on LM */
#define FXS_LINE      0     /**< First line number used in application space */
#define FXO_LINE      1     /**< Second line number used in application space */

/*
 * Function used to connect FXS line to FXO line (when timeslots are set as 0/1)
 */
EXTERN int HighwayALoopBack(void);

/*
 * Application memory space for the API required Device/Line contexts and
 * objects.  This application uses the simplest memory scheme -- global, but the
 * API will work with any memory management scheme.
 */
VpDevCtxType pDevCtx[NUM_DEVICES];
VpLineCtxType pLineCtx[NUM_LINES];
Vp880DeviceObjectType pDevObj[NUM_DEVICES];
Vp880LineObjectType pLineObj[NUM_LINES];

static VpDeviceIdType fxoDeviceId, fxsDeviceId;

/* Enum type defining possible FXS line states */
typedef enum {
    FXS_IDLE,       /**< Line is neither off-hook, nor Ringing */
    FXS_RINGING,    /**< Line is in a Ringing (cadenced) state */
    FXS_LOOP_CLOSE, /**< Line is detecting Loop Close and should be set to Line
                     * Talk
                     */
    FXS_DISCONNECT, /**< Line is in disconnect state */
    FXS_LINES_ENUM_SIZE = FORCE_STANDARD_C_ENUM_SIZE /* Portability Req. */
} FXSLineStates;

/* Enum type defining possible FXO line states */
typedef enum {
    FXO_IDLE,       /**< FXO is providing a loop open with OHT */
    FXO_LOOP_CLOSE, /**< FXO is providing a loop close */
    FXO_LINES_ENUM_SIZE = FORCE_STANDARD_C_ENUM_SIZE /* Portability Req. */
} FXOLineStates;

/**
 * Function:  InitQuickStartSettings()
 *
 * Description:  Function used to initialize the device and lines of the device
 * to settings not provided by default in the API.  This function should be the
 * first function called after a device init complete event occurs.
 */
void
InitQuickStartSettings(
    VpEventType *pEvent)
{
    VpDevCtxType *pDevCtx = pEvent->pDevCtx;
    VpDeviceInfoType deviceInfo;
    uint16 handle = 0;

    VpOptionDeviceIoType devIoOption;
    VpDeviceIoAccessDataType devIoData;

    /*
     * API-II structure instance that is setup for tx and rx timeslots used to
     * set the FXS line to timeslot 0, and FXO line to timeslot 1
     */
    VpOptionTimeslotType timeSlots;

    deviceInfo.pDevCtx = pDevCtx;
    deviceInfo.pLineCtx = VP_NULL;

    VpGetDeviceInfo(&deviceInfo);

    if (deviceInfo.deviceId == fxsDeviceId) {
        VpInitRing(&pLineCtx[FXS_LINE], RING_CAD_STD, VP_PTABLE_NULL);

        /* Set the FXS line is on Timeslot 0, and FXO in Timeslot 1 */
        timeSlots.tx = 0;
        timeSlots.rx = 0;
        VpSetOption(&pLineCtx[FXS_LINE], VP_NULL, VP_OPTION_ID_TIMESLOT,
            &timeSlots);
        VpSetLineState(&pLineCtx[FXS_LINE], VP_LINE_STANDBY);

        /* First IO as Output all others Input. Output set to Driven Mode */
        devIoOption.directionPins_31_0 = 1;
        devIoOption.directionPins_63_32 = 0;
        devIoOption.outputTypePins_31_0 = 0;
        devIoOption.outputTypePins_63_32 = 0;

        VpSetOption(VP_NULL, pEvent->pDevCtx, VP_DEVICE_OPTION_ID_DEVICE_IO,
            &devIoOption);

        /* Then make sure the IO line 1 is set to 0 */
        devIoData.accessType = VP_DEVICE_IO_WRITE;
        devIoData.accessMask_31_0 = 1;
        devIoData.accessMask_63_32 = 0;
        devIoData.deviceIOData_31_0 = 0;
        devIoData.deviceIOData_63_32 = 0;
        VpDeviceIoAccess(pEvent->pDevCtx, &devIoData);
    } else if (deviceInfo.deviceId == fxoDeviceId) {
        timeSlots.tx = 1;
        timeSlots.rx = 1;
        VpSetOption(&pLineCtx[FXO_LINE], VP_NULL, VP_OPTION_ID_TIMESLOT,
            &timeSlots);
        VpSetLineState(&pLineCtx[FXO_LINE], VP_LINE_FXO_LOOP_OPEN);
    } else {
        /*
         * Shouldn't be here, but this demonstrates proper error checking for
         * a known deviceId
         */
    }

    /*
     * Read the event mask contents so we can unmask only the FXS signaling
     * events and the FXO events. The device that the event returns for will
     * indicate which unmasking that needs to occur.
     */
    VpGetOption(VP_NULL, pDevCtx, VP_OPTION_ID_EVENT_MASK, handle);

    return;
}

/**
 * Function:  InitReadOptions()
 *
 * Description:  Function used to initialize the device event masks to only
 * allow those used by this application.
 */
void
InitReadOptions(
    VpEventType *pEvent)
{
    VpDevCtxType *pDevCtx = pEvent->pDevCtx;
    VpOptionEventMaskType eventMask;

    VpGetResults(pEvent, &eventMask);

    eventMask.signaling &= ~(VP_LINE_EVID_HOOK_OFF | VP_LINE_EVID_HOOK_ON);
    eventMask.fxo &= ~(VP_LINE_EVID_RING_ON | VP_LINE_EVID_RING_OFF);
    eventMask.fxo &= ~(VP_LINE_EVID_DISCONNECT | VP_LINE_EVID_RECONNECT);

    VpSetOption(VP_NULL, pDevCtx, VP_OPTION_ID_EVENT_MASK, &eventMask);
}

/**
 * Function:  alarmHandle()
 *
 * Description: This function is called at a periodic rate to perform all
 * required API operations for CSLAC devices.  It implements the functional
 * requirements of the application mentioned in the header.
 */
void alarmHandle(int val)
{
    /* Initialize both fxo and fxs lines to their respective idle states */
    static FXSLineStates fxsState = FXS_IDLE;
    static FXOLineStates fxoState = FXO_IDLE;

    int deviceNum;  /* deviceNum used to index through both devices on LM */

    /*
     * Initialize to mean no current event.  VpApiTick will set to FALSE if
     * there is no event, or TRUE if there is an event
     */
    bool deviceEventStatus = FALSE;

    /*
     * If there is an event, need to pass the address of an instantiated
     * structure to VpGetEvent
     */
    VpEventType pEvent;

    /* Index through (both) devices and check for active event */
    for (deviceNum = 0; deviceNum < NUM_DEVICES; deviceNum++) {
        VpApiTick(&pDevCtx[deviceNum], &deviceEventStatus);

        /*
         * If there is an active event, parse the event catagory and id data to
         * determine action
         */
        if(deviceEventStatus == TRUE) {
            while(VpGetEvent(&pDevCtx[deviceNum], &pEvent)) {
                /*
                 * If the event was a Response:Device Init Complete event,
                 * we need to initialize the device and associated lines, so
                 * send the event structure (pointer) to the Init function for
                 * handling.  Otherwise, it's an event we can process.
                 */
                if (pEvent.eventCategory == VP_EVCAT_RESPONSE) {
                    switch(pEvent.eventId) {
                        case VP_DEV_EVID_DEV_INIT_CMP:
                            InitQuickStartSettings(&pEvent);
                            break;

                        case VP_LINE_EVID_RD_OPTION:
                            InitReadOptions(&pEvent);
                            break;

                        default:
                            break;
                    }
                    /* We're initializing if a response event occurred. */
                    return;
                }

                /*
                 * The switch/case statements below implement a very simple
                 * FXO-to-FXS connected state machine.  If the FXS line goes
                 * off-hook, the FXO line is set to loop close.  If the FXO line
                 * detects ringing, the FXS line is set to Ringing (and so on).
                 */
                switch(pEvent.eventCategory) {
                    case VP_EVCAT_SIGNALING:
                        switch (pEvent.eventId) {
                            case VP_LINE_EVID_HOOK_OFF:
                                VpSetLineState(&pLineCtx[FXS_LINE],
                                    VP_LINE_TALK);
                                VpSetLineState(&pLineCtx[FXO_LINE],
                                    VP_LINE_FXO_TALK);
                                fxoState = FXO_LOOP_CLOSE;
                                break;

                            case VP_LINE_EVID_HOOK_ON:
                                if (fxsState != FXS_DISCONNECT) {
                                    VpSetLineState(&pLineCtx[FXS_LINE],
                                        VP_LINE_OHT);
                                    VpSetLineState(&pLineCtx[FXO_LINE],
                                        VP_LINE_FXO_OHT);
                                    fxoState = FXO_IDLE;
                                }
                                break;

                            default:
                                break;
                        }
                        break;

                    case VP_EVCAT_FXO:
                        switch (pEvent.eventId) {
                            case VP_LINE_EVID_RING_ON:
                                if (fxoState != FXO_LOOP_CLOSE) {
                                    VpSetLineState(&pLineCtx[FXS_LINE],
                                        VP_LINE_RINGING);
                                    fxsState = FXS_RINGING;
                                }
                                break;

                            case VP_LINE_EVID_DISCONNECT:
                                VpSetLineState(&pLineCtx[FXS_LINE],
                                    VP_LINE_DISCONNECT);
                                fxsState = FXS_DISCONNECT;
                                break;

                            case VP_LINE_EVID_RECONNECT:
                                if (fxoState == FXO_IDLE) {
                                    VpSetLineState(&pLineCtx[FXS_LINE],
                                        VP_LINE_OHT);
                                    fxsState = FXS_IDLE;
                                } else {
                                    VpSetLineState(&pLineCtx[FXS_LINE],
                                        VP_LINE_TALK);
                                    fxsState = FXS_LOOP_CLOSE;
                                }
                                break;

                            case VP_LINE_EVID_RING_OFF:
                                /*
                                 * If this did not occur as a result of being
                                 * put into loop close (i.e., the ringing just
                                 * stopped), then set the FXS line to OHT.
                                 * Otherwise, the FXS line put this line into
                                 * loop close, so do nothing.
                                 */
                                if (fxoState != FXO_LOOP_CLOSE) {
                                    VpSetLineState(&pLineCtx[FXS_LINE],
                                        VP_LINE_OHT);
                                    fxsState = FXS_IDLE;
                                }
                                break;
                        }
                        break;

                    default:
                        break;
                }
            }
        }
    }
    return;
}

/**
 * Function:  UVB_Brd_Init()
 *
 * Description: This function sets the PCLK frequencies of the UVB and connects
 * the I/O data to the small and DIN conectors.  The user will only need this
 * function if this application is running on the UVB.  Otherwise it should be
 * replaced with an equivalent hardware setup.
 */
int UVB_Brd_Init(void)
{
    int reg_offs_freq = 0;
    int reg_offs_connect = 6;
    int mask = 0xffff;
    int value_freq = 0x0660;
    int value_connect = 0xC;

    if(CPLD_Write( reg_offs_freq, mask, value_freq ) < 0) {
        return -1;
    }

    if(CPLD_Write( reg_offs_connect, mask, value_connect ) < 0) {
        return -1;
    }

    return 0;
}

/**
 * Function:  main()
 *
 * Description: This function initializes the API and Devices/Lines used on the
 * -0823 LM.  It also starts the "alarmHandle" function to be called at a
 * periodic rate.
 */
int main(void)
{
    VpStatusType status;

    /*
     * Linux specific timer structures used to create a function call at
     * specified intervals
     */
    struct itimerval timeStruct, timeOldVal;

    int deviceNum;  /* Convient variable used to sequence through the devices */
    VpDeviceIdType deviceId;

    /*
     * The following structure is specific to Linux.  It sets up a timed call to
     * a user specified function.  For the purpose of this demo, the timing is
     * 5mS and the function that will be called is "alarmHandle".  The timer
     * and function call does not start until "setitimer()" and "signal()" are
     * called (below).
     */
    timeStruct.it_interval.tv_sec = 0;
    timeStruct.it_interval.tv_usec = 5000;
    timeStruct.it_value.tv_sec = 0;
    timeStruct.it_value.tv_usec = 5000;

    timeOldVal.it_interval.tv_sec = 0;
    timeOldVal.it_interval.tv_usec = 0;
    timeOldVal.it_value.tv_sec = 0;
    timeOldVal.it_value.tv_usec = 0;

    /*
     * UVB_Brd_Int() is an initialization specific to the UVB board that sets
     * the UVB PCLK frequencies (to 8.192MHz) and connects the digital signals
     * to both the small and DIN connectors.
     */
    if(UVB_Brd_Init() < 0) {
        return -1;
    };

    HighwayALoopBack();

    /*
     * Simple loop to initialize the device/line contexts and objects for the
     * API as well as initialize the Devices and Lines.  When this loop is
     * complete, all devices and lines on the -0823 LM can be used.
     */
    for (deviceNum = 0; deviceNum < NUM_DEVICES; deviceNum++) {
        deviceId = ((deviceNum << 3) | MPI_3WIRE);
        VpMakeDeviceObject(VP_DEV_880_SERIES, deviceId, &pDevCtx[deviceNum],
            &pDevObj[deviceNum]);

        if (deviceNum == FXS_DEVICE) {
            VpMakeLineObject(VP_TERM_FXS_GENERIC, 0, &pLineCtx[FXS_LINE],
                &pLineObj[FXS_LINE], &pDevCtx[FXS_DEVICE]);
            fxsDeviceId = deviceId;
        } else {
            VpMakeLineObject(VP_TERM_FXO_GENERIC, 0, &pLineCtx[FXO_LINE],
                &pLineObj[FXO_LINE], &pDevCtx[FXO_DEVICE]);
            fxoDeviceId = deviceId;
        }

        /* Initialize the Device */
        status =
        VpInitDevice(&pDevCtx[deviceNum], DEV_PROFILE, AC_600R_FXS, DC_25MA_CC,
            RING_20HZ_SINE, AC_600R_FXO, FXO_DIALING);

        if (status != VP_STATUS_SUCCESS) {
            deviceId = ((deviceNum << 3) | MPI_4WIRE);
            VpMakeDeviceObject(VP_DEV_880_SERIES, deviceId, &pDevCtx[deviceNum],
                &pDevObj[deviceNum]);

            if (deviceNum == FXS_DEVICE) {
                VpMakeLineObject(VP_TERM_FXS_GENERIC, 0, &pLineCtx[FXS_LINE],
                    &pLineObj[FXS_LINE], &pDevCtx[FXS_DEVICE]);
                fxsDeviceId = deviceId;
            } else {
                VpMakeLineObject(VP_TERM_FXO_GENERIC, 0, &pLineCtx[FXO_LINE],
                    &pLineObj[FXO_LINE], &pDevCtx[FXO_DEVICE]);
                fxoDeviceId = deviceId;
            }

            /* Initialize the Device */
            status =
            VpInitDevice(&pDevCtx[deviceNum], DEV_PROFILE, AC_600R_FXS,
                DC_25MA_CC, RING_20HZ_SINE, AC_600R_FXO, FXO_DIALING);

            if (status != VP_STATUS_SUCCESS) {
                printf("\n\rDevice Did Not Initialize\n\r");
                fflush(NULL);
                exit(-1);
            } else {
                printf("\n\rDevice Initialized in MPI 4-Wire Mode\n\r");
                fflush(NULL);
            }
        } else {
            printf("\n\rDevice Initialized in MPI 3-Wire Mode\n\r");
            fflush(NULL);
        }
    }

    /*
     * Now create the loopback from timeslot 0 tx to timeslot 1 rx and
     * visa-versa
     */
    HighwayALoopBack();

    /*
     * The following two function calls were described above as Linux specific.
     * They are used to call a user defined function at set increments.  The
     * user may have several other methods to perform the same.
     */
    signal(SIGALRM, alarmHandle);
    setitimer(ITIMER_REAL, &timeStruct, &timeOldVal);

    /*
     * Once the "tick" is setup, the main program can enter a "while(1)" loop
     * and the user specified function will take over the application (with the
     * help of the Linux OS)
     */
    while(1);
}


