/*
 * cli.c
 *
 *  This file contains Caller ID FSK Module for the VP-SDK
 *
 *
 * Copyright (c) 2000, Zarlink Semiconductor Inc.
 * All rights reserved
 *
 * This software is the property of Zarlink Semiconductor , Inc. Please refer to the
 * Non Disclosure Agreement (NDA) that you have signed for more information
 * on legal obligations in using, modifying or distributing this file.
 */

#include "vp_api_types.h"
#include "cli.h"
#include <string.h>
#include <stdlib.h>

/*------------------------------------------------------------------------------
 * AppCreateBuffer()
 *  Creates a SDMF or a MDMF Caller ID Message for U.S. Caller ID Systems.
 *  This can also be used to send simple U.K. Caller ID messages, assuming that
 *  the name field is not empty or null pointer (it could be " ", though).
 *  This function is expected to be customized for customer applications.
 * Preconditions
 *  If number is an empty string, or a null pointer, then the message
 *  format will be SDMF. Otherwise, MDMF is used.
 *  It is assumed that month, day, hour, and minute strings are all of size 2.
 *  In other words, they should all have two ascii characters starting at array
 *  position 0.
 *  It is assumed that name is a null terminated string.
 *  It is assumed that appBuffer is large enough to contains the completed
 *  message field (Minus the checksum). This size should be at least:
 *   If SDMF = 12
 *   If MDMF = 18 + sizeof(name parameter)
 * Postconditions
 *  appBuffer contains a complete message layer message. The buffer contains
 * the following format:
 *  BYTE #s  |  BYTE DESCRIPTION
 *  0        |  Message Type Identifier (either SDMF or MDMF)
 *  1        |  Message Length
 *  2        |  Date & Time Parameter Indicator (only present if using MDMF)
 *  3        |  Date & Time Parameter Length    (only present if using MDMF)
 *  4-5      |  Month String
 *  6-7      |  Day String
 *  8-9      |  Hour String
 *  10-11    |  Minute String
 *  12       |  Numer Parameter Indicator (only present if using MDMF)
 *  13       |  Number Parameter Length - 0x0A (only present if using MDMF)
 *  14-23    |  Phone number data string
 *  15       |  Name Parameter Indicator (only present if using MDMF)
 *  16       |  Name Parameter Length (only present if using MDMF)
 *  17 - X   |  Name Parameter Data (only present if using MDMF)
 *  X+1      |  Null Terminating Character (to indicate end of data)
 *----------------------------------------------------------------------------*/
void
AppCreateBuffer(
    char* month,        /* Two ASCII characters representing the month */
    char* day,          /* Two ASCII characters representing the day of the month */
    char* hour,         /* Two ASCII chars representing the hour in military time */
    char* minute,       /* Two ASCII chars representing the minute of the hour */
    uint16 number,       /* Source phone number (Three digits) */
    char* name,         /* Null terminated string representing the caller's name */
    uint8* appBuffer    /* Buffer to contain the completed Application Buffer */
    )
{
    bool mdmf = TRUE;   /* Indicates a multiple data message format */
    uint8 appIndex = 2; /* Start populating after messagetype and length */
    uint8 marker = 0;
    uint8 fIndex=0;

    if ( (name==NULL) || (name[0]=='\0') ) {
        mdmf = FALSE;
    }

    /* Populate the date and time fields */
    if (mdmf) {
        appBuffer[appIndex++] = CLI_PARAM_DATETIME; /* Parameter type */
        appBuffer[appIndex++] = 0x08;               /* Parameter Length */
    }
    appBuffer[appIndex++] = month[0];
    appBuffer[appIndex++] = month[1];
    appBuffer[appIndex++] = day[0];
    appBuffer[appIndex++] = day[1];
    appBuffer[appIndex++] = hour[0];
    appBuffer[appIndex++] = hour[1];
    appBuffer[appIndex++] = minute[0];
    appBuffer[appIndex++] = minute[1];

    /* Populate the Number field */
    if (mdmf) {
        appBuffer[appIndex++] = CLI_PARAM_NUMBER;   /* Parameter type */
        appBuffer[appIndex++] = 0x0A;               /* Parameter Length */
    }
    strcpy((char*)&appBuffer[appIndex],"5555550");
    strcpy((char*)&appBuffer[appIndex+7],"123");
    appIndex += 10;

    /* Populate the Name field */
    if (mdmf) {
        appBuffer[appIndex++] = CLI_PARAM_NAME;     /* Parameter type */
        marker = appIndex;                          /* Save length field location */
        appBuffer[appIndex++] = 0x0E;               /* Parameter Length */
    }
    while (name[fIndex]!='\0') {
        appBuffer[appIndex++] = name[fIndex++];
    }
    if (mdmf) {
        appBuffer[marker] = appIndex-marker-1;      /* Correct the length field */
    }

    /* Null terminate the string */
    appBuffer[appIndex] = '\0';

    /* Fill in message type and message length fields */
    if (mdmf) {
        appBuffer[0] = CLI_MDMF;
    } else {
        appBuffer[0] = CLI_SDMF;
    }
    appBuffer[1] = appIndex - 2; /* Correct for length and message type exclusion */

    return;
} /* AppCreateBuffer() */
