/** \file qs_le71hr0827_dat.c
 * qs_le71hr0827_dat.c
 *
 *  This file is a Quick-Start application for the LE71HR0827 Line module.  It
 * accompanies a profiles8.c and profiles8.h file set to configure the device
 * and lines on the -0827 Line Module.  The purpose of this applicaiton is to
 * demonstrate the capabilities of the VoicePort chipset via the API w.r.t Tone
 * Generation, DTMF Generation, and Dial Pulse Detection.
 *
 * Copyright (c) 2006, Zarlink Semiconductor Inc.
 * All rights reserved
 *
 * This software is the property of Zarlink Semiconductor , Inc. Please refer to the
 * Non Disclosure Agreement (NDA) that you have signed for more information
 * on legal obligations in using, modifying or distributing this file.
 */

/*
 * These headers are for UVB/Linux purposes only.  The user will not find them
 * in the API
 */
#include <sys/time.h>
#include <signal.h>
#include "uvb_xs.h"

#include <stdio.h>

/*
 * These files are required by all applications.  "vp_api.h" includes all of the
 * API libraries required by the application.  "profiles8.h" includes the
 * profiles used by the application.  At a minimum, there must be a Device
 * Profile for the device to initialize.
 */
#include "vp_api.h"
#include "profiles8.h"

/* Application specific #define that improve code readability */
#define NUM_LINES     4
#define NUM_DEVICES   2
#define LINES_PER_DEVICE 2

/* UVB specific values for device ID depending on the MPI Mode */
#define MPI_3WIRE 0x30
#define MPI_4WIRE 0x10

/*
 * Application memory space for the API required Device/Line contexts and
 * objects.  This application uses the simplest memory scheme -- global, but the
 * API will work with any memory management scheme.
 */
VpDevCtxType pDevCtx[NUM_DEVICES];
VpLineCtxType pLineCtx[NUM_LINES];
Vp880DeviceObjectType pDevObj[NUM_DEVICES];
Vp880LineObjectType pLineObj[NUM_LINES];

/*
 * Use this to hold the state of the lines so we can toggle Standby/Ringing
 * between each On-Hook state.
 */
bool fxsOnHook[NUM_LINES] = {FALSE, FALSE, FALSE, FALSE};

/**
 * Function:  InitQuickStartSettings()
 *
 * Description:  Function used to initialize the device and lines of the device
 * to settings not provided by default in the API.  This function should be the
 * first function called after a device init complete event occurs.
 */
void
InitQuickStartSettings(
    VpEventType *pEvent)
{
    VpDevCtxType *pDevCtx = pEvent->pDevCtx;
    uint8 chanNum;
    uint16 handle = 0;

    /*
     * For this demo, we want the API to detect the dial pulses on the FXS
     * line, then initialize the ringing cadence.
     */
    VpOptionPulseModeType pulseMode = VP_OPTION_PULSE_DECODE_ON;
    VpSetOption(VP_NULL, pDevCtx, VP_OPTION_ID_PULSE_MODE, &pulseMode);

    for (chanNum = 0; chanNum < NUM_LINES; chanNum++) {
        VpInitRing(&pLineCtx[chanNum], RING_CAD_STD, VP_PTABLE_NULL);

        fxsOnHook[chanNum] = FALSE;
        VpSetLineState(&pLineCtx[chanNum], VP_LINE_STANDBY);
    }

    /*
     * Read the event mask contents so we can unmask only the FXS signaling
     * events.
     */
    VpGetOption(VP_NULL, pDevCtx, VP_OPTION_ID_EVENT_MASK, handle);

    return;
}

/**
 * Function:  InitReadOptions()
 *
 * Description:  Function used to initialize the device event masks to only
 * allow those used by this application.
 */
void
InitReadOptions(
    VpEventType *pEvent)
{
    VpDevCtxType *pDevCtx = pEvent->pDevCtx;
    VpOptionEventMaskType eventMask;

    VpGetResults(pEvent, &eventMask);

    eventMask.signaling &= ~(VP_LINE_EVID_HOOK_OFF | VP_LINE_EVID_HOOK_ON);
    eventMask.signaling &= ~(VP_LINE_EVID_FLASH | VP_LINE_EVID_STARTPULSE);
    eventMask.signaling &= ~(VP_LINE_EVID_PULSE_DIG);

    VpSetOption(VP_NULL, pDevCtx, VP_OPTION_ID_EVENT_MASK, &eventMask);
}

/**
 * Function:  ProcessFXSEvent()
 *
 * Description:  Function used to perform some action on the FXS lines when an
 * event occurs.
 */
void
ProcessFXSEvent(
    VpEventType *pEvent)
{
    VpEventCategoryType eventCategory = pEvent->eventCategory;
    uint16 evDat = pEvent->eventData;
    uint16 eventId = pEvent->eventId;
    VpLineCtxType *pLineCtx = pEvent->pLineCtx;
    uint8 chanId = pEvent->channelId;
    /*
     * The tone profiles (pointers) in the following list are created from the
     * Profile Wizard
     */
    VpProfilePtrType tonePtrs[] = {
        TONE_DIAL, TONE_DIAL, TONE_BUSY, TONE_RINGBACK,
        TONE_ROH, TONE_ROH, TONE_ROH
    };

    /*
     * The cadence profiles (pointers) in the following list are created from
     * the Profile Wizard
     */
    VpProfilePtrType cadencePtrs[] = {
        TONE_CAD_DIAL0, TONE_CAD_DIAL1, TONE_CAD_BUSY0, TONE_CAD_RNGBACK,
        TONE_CAD_ROH, TONE_CAD_UK_HOWL, TONE_CAD_AUS_HWL
    };

    /*
     * Ignore all other type of events except signaling on the FXS device.  For
     * those, we will look at the data and determine the digit (or hook, or
     * flash) that caused the event.  Depending on the specific event, we will
     * send a tone or set ringing on the line.
     *
     * Note:  We specified the Pulse digit generated from the FXO line via the
     * profiles, not via Options.
     */

    switch(eventCategory) {
        case VP_EVCAT_SIGNALING:
            switch (eventId) {
                case VP_LINE_EVID_HOOK_OFF:
                    VpSetLineState(pLineCtx, VP_LINE_TALK);
                    if (fxsOnHook[chanId] == TRUE) {
                        VpSetLineTone(pLineCtx, VP_PTABLE_NULL, VP_PTABLE_NULL,
                            VP_NULL);
                    } else {
                        VpSetLineTone(pLineCtx, tonePtrs[0], cadencePtrs[0],
                            VP_NULL);
                    }
                    break;

                case VP_LINE_EVID_HOOK_ON:
                    VpSetLineTone(pLineCtx, VP_PTABLE_NULL, VP_PTABLE_NULL,
                        VP_NULL);

                    if (fxsOnHook[chanId] == FALSE) {
                        fxsOnHook[chanId] = TRUE;
                        VpSetLineState(pLineCtx, VP_LINE_RINGING);
                    } else {
                        fxsOnHook[chanId] = FALSE;
                        VpSetLineState(pLineCtx, VP_LINE_STANDBY);
                    }
                    break;

                case VP_LINE_EVID_PULSE_DIG:
                    VpSetLineState(pLineCtx, VP_LINE_TALK);

                    if (pEvent->eventData <= 7) {
                        VpSetLineTone(pLineCtx, tonePtrs[evDat-1],
                            cadencePtrs[evDat-1], VP_NULL);
                    }
                    break;

                case VP_LINE_EVID_STARTPULSE:
                    VpSetLineTone(pLineCtx, VP_PTABLE_NULL, VP_PTABLE_NULL,
                        VP_NULL);
                    break;

                case VP_LINE_EVID_FLASH:
                    /* Could do something here */
                    break;

                default:
                    break;
            }
            break;

        default:
            break;
    }
}

/**
 * Function:  alarmHandle()
 *
 * Description: This function is called at a periodic rate to perform all
 * required API operations.  It implements the functional requirements of the
 * application mentioned in the header.
 */
void alarmHandle(int val)
{
    int deviceNum;
    bool deviceEventStatus = FALSE;
    VpEventType pEvent;

    /*
     * This loop will query the FXS device for events, and when an event is
     * found (deviceEventStatus = TRUE), it will parse the event and perform
     * further operations.
     */
    for (deviceNum = 0; deviceNum < NUM_DEVICES; deviceNum++) {
        VpApiTick(&pDevCtx[deviceNum], &deviceEventStatus);
        if(deviceEventStatus == TRUE) {
            while(VpGetEvent(&pDevCtx[deviceNum], &pEvent)) {
                /*
                 * If the event was a Response:Device Init Complete event,
                 * we need to initialize the device and associated lines, so
                 * send the event structure (pointer) to the Init function for
                 * handling.
                 */
                if (pEvent.eventCategory == VP_EVCAT_RESPONSE) {
                    if(pEvent.eventId == VP_DEV_EVID_DEV_INIT_CMP) {
                        InitQuickStartSettings(&pEvent);
                    } else if (pEvent.eventId == VP_LINE_EVID_RD_OPTION) {
                        InitReadOptions(&pEvent);
                    } else {
                        /* Do nothing */
                    }
                    /* We're initializing if a response event occurred. */
                    return;
                }
                ProcessFXSEvent(&pEvent);
            }
        }
    }
    return;
}

/**
 * Function:  UVB_Brd_Init()
 *
 * Description: This function sets the PCLK frequencies of the UVB and connects
 * the I/O data to the small and DIN conectors.  The user will only need this
 * function if this application is running on the UVB.  Otherwise it should be
 * replaced with an equivalent hardware setup.
 */
int UVB_Brd_Init(void)
{
    int reg_offs_freq = 0;
    int reg_offs_connect = 6;
    int mask = 0xffff;
    int value_freq = 0x0660;
    int value_connect = 0xC;

    if(CPLD_Write( reg_offs_freq, mask, value_freq ) < 0) {
        return -1;
    }

    if(CPLD_Write( reg_offs_connect, mask, value_connect ) < 0) {
        return -1;
    }

    return 0;
}

/**
 * Function:  main()
 *
 * Description: This function initializes the API and Devices/Lines used on the
 * -0824 LM.  It also starts the "alarmHandle" function to be called at a
 * periodic rate.
 */
int main(void)
{
    struct itimerval timeStruct, timeOldVal;
    int deviceNum, chanNum;

    VpStatusType status;

    /*
     * The following structure is specific to Linux.  It sets up a timed call
     * to a user specified function.  For the purpose of this demo, the timing
     * is 5mS and the function that will be called is "alarmHandle".  The timer
     * and function call does not start until "setitimer()" and "signal()" are
     * called (below).
     */
    timeStruct.it_interval.tv_sec = 0;
    timeStruct.it_interval.tv_usec = 5000;
    timeStruct.it_value.tv_sec = 0;
    timeStruct.it_value.tv_usec = 5000;

    timeOldVal.it_interval.tv_sec = 0;
    timeOldVal.it_interval.tv_usec = 0;
    timeOldVal.it_value.tv_sec = 0;
    timeOldVal.it_value.tv_usec = 0;

    /*
     * UVB_Brd_Int() is an initialization specific to the UVB board that sets
     * the UVB PCLK frequencies (to 8.192MHz) and connects the digital signals
     * to both the small and DIN connectors.
     */
    if(UVB_Brd_Init() < 0) {
        return -1;
    };

    /*
     * Simple loop to initialize the device/line contexts and objects for the
     * API as well as initialize the Devices and Lines.  When this loop is
     * complete, all devices and lines on the -0824 LM can be used.
     */
    for (deviceNum = 0; deviceNum < NUM_DEVICES; deviceNum++) {
        VpMakeDeviceObject(VP_DEV_880_SERIES, ((deviceNum << 3) | MPI_4WIRE),
            &pDevCtx[deviceNum], &pDevObj[deviceNum]);

        if (deviceNum == 0) {
            for (chanNum = 0; chanNum < LINES_PER_DEVICE; chanNum++) {
                VpMakeLineObject(VP_TERM_FXS_ISOLATE, chanNum, &pLineCtx[chanNum],
                    &pLineObj[chanNum], &pDevCtx[deviceNum]);
            }
        } else {
            for (chanNum = 0; chanNum < LINES_PER_DEVICE; chanNum++) {
                VpMakeLineObject(VP_TERM_FXS_ISOLATE, chanNum,
                    &pLineCtx[chanNum+LINES_PER_DEVICE],
                    &pLineObj[chanNum+LINES_PER_DEVICE], &pDevCtx[deviceNum]);
            }
        }

        /* Initialize the Device */
        status = VpInitDevice(&pDevCtx[deviceNum], DEV_PROFILE, AC_600R_FXS,
            DC_25MA_CC, RING_20HZ_SINE, VP_PTABLE_NULL, VP_PTABLE_NULL);

        if (status != VP_STATUS_SUCCESS) {
            /* Try MPI 3-Wire Mode if MPI 4-Wire Failed */
            VpMakeDeviceObject(VP_DEV_880_SERIES, ((deviceNum << 3) | MPI_3WIRE),
                &pDevCtx[deviceNum], &pDevObj[deviceNum]);

            if (deviceNum == 0) {
                for (chanNum = 0; chanNum < LINES_PER_DEVICE; chanNum++) {
                    VpMakeLineObject(VP_TERM_FXS_ISOLATE, chanNum,
                        &pLineCtx[chanNum], &pLineObj[chanNum], &pDevCtx[deviceNum]);
                }
            } else {
                for (chanNum = 0; chanNum < LINES_PER_DEVICE; chanNum++) {
                    VpMakeLineObject(VP_TERM_FXS_ISOLATE, chanNum,
                        &pLineCtx[chanNum+LINES_PER_DEVICE],
                        &pLineObj[chanNum+LINES_PER_DEVICE], &pDevCtx[deviceNum]);
                }
            }

            /* Re-attempt to initialize the Device */
            status = VpInitDevice(&pDevCtx[deviceNum], DEV_PROFILE, AC_600R_FXS,
                DC_25MA_CC, RING_20HZ_SINE, VP_PTABLE_NULL, VP_PTABLE_NULL);

            if (status != VP_STATUS_SUCCESS) {
                printf("\n\rDevice %d Did Not Init Properly\t%d\n\r",
                    deviceNum, status);
                fflush(NULL);
            } else {
                printf("\n\rDevice %d Initialized in MPI 3-Wire Mode\n\r",
                    deviceNum);
                fflush(NULL);
            }
        } else {
            printf("\n\rDevice %d Initialized in MPI 4-Wire Mode\n\r",
                deviceNum);
            fflush(NULL);
        }
    }

    /*
     * The following two function calls were described above as Linux specific.
     * They are used to call a user defined function at set increments.  The
     * user may have several other methods to perform the same.
     */
    signal(SIGALRM, alarmHandle);
    setitimer(ITIMER_REAL, &timeStruct, &timeOldVal);

    /*
     * Once the "tick" is setup, the main program can enter a "while(1)" loop
     * and the user specified function will take over the application (with the
     * help of the Linux OS)
     */
    while(1);
}



