/** \file sys_service.c
 * sys_service.c
 *
 *  This file implements the required system services for the API-II using a
 * Linux OS running on the UVB.  The user should replace the functions provided
 * here with the equivalent based on their OS and hardware.
 *
 * Copyright (c) 2008, Zarlink Semiconductor Inc.
 * All rights reserved
 *
 * This software is the property of Zarlink Semiconductor , Inc. Please refer to the
 * Non Disclosure Agreement (NDA) that you have signed for more information
 * on legal obligations in using, modifying or distributing this file.
 */
/* #define __USE_GNU */
/* #include <sys/unistd.h> */
#include <sys/types.h>
#include <sys/ipc.h>
#include <sys/sem.h>
#include <sys/stat.h>
#include <errno.h>
#include <unistd.h>
#include <string.h>
#include <signal.h>
#include <stdlib.h>     /* exit() */
#include "vp_api_cfg.h"
#include "vp_api_types.h"
#include "sys_service.h"
#include "vpdlib.h"     /* This file is necessary only for Zarlink Semiconductor UVB */

#undef SYS_SERVICE_DEBUG
#ifdef SYS_SERVICE_DEBUG
 #include <stdio.h>
 #define dprintf for (i=0; i<pid%5; i++) {printf("\t");};printf
#else
 #define dprintf fake_printf
#endif

static const key_t key = 0xADEF;
static const int flagMakeSem = S_IRUSR | S_IWUSR | S_IROTH | S_IWOTH | IPC_CREAT | IPC_EXCL;
static const int flagGetSem =  S_IRUSR | S_IWUSR | S_IROTH | S_IWOTH;
static const int NUM_SEMS = 1; /*VP_NUM_DEVICES * VP_NUM_CRITICAL_SEC_TYPES;*/
static int critDepth = 0;

VpTestHeapType testHeap;


#define PERM_UREAD      0400
#define PERM_UWRITE     0200
#define PERM_GREAD      0040
#define PERM_GWRITE     0020
#define PERM_OREAD      0004
#define PERM_OWRITE     0002

int fake_printf(const char *format, ...) {
	/* don't do anything */
    return 0;
}

/*
 * VpSysWait() function implementation is needed only for CSLAC devices
 * (880, 790). For other devices this function could be commented.
 */
void
VpSysWait(
    uint8 time)  /* Time specified in increments of 125uS (e.g. 4 = 500uS) */
{
    usleep(125 * time);
}

void VpSysSemaphoreInit(int semaphoreId)
{
	int i;
	int rc;
	unsigned short initialValues[NUM_SEMS];
	union semun semArg;
	pid_t pid = 0;
#ifdef SYS_SERVICE_DEBUG
	pid = getpid();
#endif
	for (i=0; i < NUM_SEMS; i++) {
		initialValues[i] = 1;
	}
	semArg.array = initialValues;

	dprintf("%u:initing semaphore key=0x%X, id=0x%X...",pid, key, semaphoreId);
	if ((rc = semctl(semaphoreId, 0, SETALL, semArg)) == -1) {
#ifdef SYS_SERVICE_DEBUG
		perror("semctl SETALL");
#endif
		exit(1);
	}
	/* dprintf("%u:VpSysSemaphoreInit finished.\n",pid); */
}

/*
 * VpSysEnterCritical(), VpSysExitCritical():
 *
 *  These functions allow for disabling interrupts while executing nonreentrant
 * portions of VoicePath API code. Note that the following implementations of
 * enter/exit critical section functions are simple implementations. These
 * functions could be expanded (if required) to handle different critical
 * section types differently.
 *
 * Params:
 *  VpDeviceIdType deviceId: Device Id (chip select ID)
 *  VpCriticalSecType: Critical section type
 *
 * Return:
 *  Number of critical sections currently entered for the device.
 */
uint8
VpSysEnterCritical(
    VpDeviceIdType deviceId,
    VpCriticalSecType criticalSecType)
{
	int rc;
	int semaphoreId;
	struct sembuf semOp[1];
	pid_t pid = 0;
#ifdef SYS_SERVICE_DEBUG
	pid = getpid();
#endif
	/* semOp[0].sem_num = deviceId*VP_NUM_CRITICAL_SEC_TYPES + criticalSecType;	*/
	semOp[0].sem_num = 0;
	semOp[0].sem_op = -1;
	semOp[0].sem_flg = SEM_UNDO;

	critDepth++;
	if (critDepth == 1) {
		if ((semaphoreId = semget(key, NUM_SEMS, flagMakeSem)) >= 0) {
 			/* we just created the semaphore and it needs to be inited */
			VpSysSemaphoreInit(semaphoreId);
			dprintf("%u:Made new semaphore %d (hopefully sem_val=1)\n", pid, semaphoreId);
 		} else {
 			/* semaphore already exists, we just need to get an id */
 			if ((semaphoreId = semget(key, NUM_SEMS, flagGetSem)) < 0) {
	    		/* something has gone sour */
#ifdef SYS_SERVICE_DEBUG
				perror("semget flagGetSem");
#endif
				exit(1);
			}
			/* dprintf("%u:Got existing semaphore %d\n",pid, semaphoreId); */
	    }

		/* dprintf("%u:?v,d=%d\n",pid);	*/
		if ((rc = semop(semaphoreId, semOp, 1)) < 0) {
#ifdef SYS_SERVICE_DEBUG
			perror("semop -1");
#endif
			exit(1);
		}
		if (rc)	{
			dprintf("semaphore error with deviceId=0x%X, criticalSecType=0x%X\n",
				deviceId, criticalSecType);
				return -1;
		}
	}
	dprintf("%u:v,d=%d\n",pid, critDepth);

	return critDepth;
} /* VpSysEnterCritical() */

uint8
VpSysExitCritical(
    VpDeviceIdType deviceId,
    VpCriticalSecType criticalSecType)
{
	int rc;
	int semaphoreId;
	struct sembuf semOp[1];
	pid_t pid = 0;
#ifdef SYS_SERVICE_DEBUG
	pid = getpid();
#endif
	/* semOp[0].sem_num = deviceId*VP_NUM_CRITICAL_SEC_TYPES + criticalSecType;	*/
	semOp[0].sem_num = 0;
	semOp[0].sem_op = 1;
	semOp[0].sem_flg = SEM_UNDO;

	critDepth--;
	if (critDepth == 0) {
		/* we can free the semaphore now */
		if ((semaphoreId = semget(key, NUM_SEMS, flagGetSem)) < 0) {
#ifdef SYS_SERVICE_DEBUG
	    	perror("semget");
#endif
	    	exit(1);
	    }
		/* dprintf("%u:Got existing semaphore %d\n",pid, semaphoreId); */
		dprintf("%u:^,d=%d\n",pid,critDepth);
		if ((rc = semop(semaphoreId, semOp, 1)) == -1) {
#ifdef SYS_SERVICE_DEBUG
			perror("semop +1");
#endif
			exit(1);
		}

		if (rc)	{
			dprintf("semaphore error with deviceId=0x%X, criticalSecType=0x%X",
				deviceId, criticalSecType);
		}
	} else {
		dprintf("%u:^,d=%d\n",pid,critDepth);
	}
	/*dprintf("%u:VpSysExitCritical finished with critDepth=%d.\n",pid, critDepth);*/

	return critDepth;
} /* VpSysExitCritical() */

/**
 * VpSysDisableInt(), VpSysEnableInt(), and VpSysTestInt()
 *
 *  These functions are used by the CSLAC device family for interrupt driven
 * polling modes. These are called by the API to detect when a non-masked
 * device status has changed.  If using SIMPLE_POLL mode, these functions do not
 * require implementation.
 *
 * Preconditions:
 *  None. The implementation of these functions is architecture dependent.
 *
 * Postconditions:
 *  VpSysDisableInt() - The interrupt associated with the deviceId passed is
 * disabled.
 *
 * VpSysEnableInt() - The interrupt associated with the deviceId passed is
 * enabled.
 *
 * VpSysTestInt() - The return value is TRUE if an interrupt occurred, otherwise
 * return FALSE.
 *
 * These functions are needed only for CSLAC devices
 * (880, 790). For other devices these functions could be commented.
 *
 */
void
VpSysDisableInt(
    VpDeviceIdType deviceId)
{
    return;
}

void
VpSysEnableInt(
    VpDeviceIdType deviceId)
{
    return;
}

bool
VpSysTestInt(
    VpDeviceIdType deviceId)

{
    return FALSE;
}

/**
 * VpSysDtmfDetEnable(), VpSysDtmfDetDisable()
 *
 *  These functions are used by the CSLAC device family for devices that do not
 * internally detect DTMF. It is used for Caller ID type-II and is provided to
 * enable external DTMF detection.
 *
 * Preconditions:
 *  None. The implementation of these functions is application dependent.
 *
 * Postconditions:
 *  VpSysDtmfDetEnable() - The device/channel resource for DTMF detection is
 * enabled.
 *
 *  VpSysDtmfDetDisable() - The device/channel resource for DTMF detection is
 * disabled.
 *
 * These functions are needed only for CSLAC devices
 * (880, 790). For other devices these functions could be commented.
 *
 */

void
VpSysDtmfDetEnable(
    VpDeviceIdType deviceId,
    uint8 channelId)
{
}

void
VpSysDtmfDetDisable(
    VpDeviceIdType deviceId,
    uint8 channelId)
{
}

/*
 * The following functions VpSysTestHeapAcquire(),  VpSysTestHeapRelease()
 * VpSysPcmCollectAndProcess() and are needed only for CSLAC devices
 * (880). For other devices these functions could be commented.
 *
 */
void *
VpSysTestHeapAcquire(
    uint8 *pHeapId)
{
	void *pTestHeap;
	int fd = VpdOpenSocket();

	/* If VoicePath Daemon is not running, initialize a heap statically
	 * (for running quick-starts) */
	if (fd == -1) {
		dprintf("The error shows that VoicePath Daemon is not running\n");
		dprintf("Allocating local memory (%d bytes) for test heap.\n\n", sizeof(testHeap));
		pTestHeap = &testHeap;
	} else {
		pTestHeap = VpdTestAcquire(fd, (int*)pHeapId);
	}

    close(fd);
    return pTestHeap;
} /* VpSysTestHeapAcquire() */

bool
VpSysTestHeapRelease(
    uint8 heapId)
{
	bool status;
	int fd = VpdOpenSocket();

	/* If VoicePath Daemon is not running */
	if (fd != -1) {
		status = VpdTestRelease(fd, (int)heapId);
		close(fd);
	} else {
		status = TRUE;
		dprintf("The error shows that VoicePath Daemon is not running\n");
		dprintf("No heap to release\n");
	}
    return status;
} /* VpSysTestHeapRelease() */

void
VpSysPcmCollectAndProcess(
    void *pLineCtx,
    VpDeviceIdType deviceId,
    uint8 channelId,
    uint8 startTimeslot,
    uint16 operationTime,
    uint16 settlingTime,
    uint16 operationMask)
{
    int fd = VpdOpenSocket();
    VpdPcmIntegrate(fd, pLineCtx, startTimeslot, settlingTime, operationTime,
        operationMask);
    close(fd);
} /* VpSysPcmCollectAndProcess() */




