/** \file vp_hal.c
 * vp_hal.c
 *
 * This file contains the platform dependent code for the Hardware Abstraction
 * Layer (HAL). This implementation is for embedded Linux. This HAL
 * implementation assumes a Linux kernel mode driver and also an application
 * level interface to those kernel drivers.
 *
 * Copyright (c) 2008, Zarlink Semiconductor Inc.
 * All rights reserved
 *
 * This software is the property of Zarlink Semiconductor , Inc. Please refer to the
 * Non Disclosure Agreement (NDA) that you have signed for more information
 * on legal obligations in using, modifying or distributing this file.
 */
#include <unistd.h>
#include <sys/types.h>
#include <stdio.h>

#include "asm/board/uvb_conn.h" /* Required for Zarlink Semiconductor UVB Only */
#include "vp_api_types.h"
#include "hbi_hal.h"
#include "hbi_hal.h"
#include "sys_service.h"
#include "uvb_xs.h"              /* Required for Zarlink Semiconductor UVB Only */
#include "vp_hal.h"

/******************************************************************************
 * NOTE: If a function is not necessary for a given device, That function can *
 * can be completely deleted or dummy implementation could be provided.       *
 ******************************************************************************/

/*****************************************************************************
 * HAL functions for VCP and VPP. Not necessary for CSLAC devices.
 ****************************************************************************/
/*
 * Define HBI_HAL_DEBUG to include parameter checking code and several
 * printf()s showing HBI activity.
 */
#undef HBI_HAL_DEBUG
#undef HBI_HAL_DEBUG_LONG
#ifdef HBI_HAL_DEBUG
#include <string.h>

/* For run-time control of HBI tracing: */
//static bool VpHbiTraceEnable = FALSE;
#endif /* HBI_HAL_DEBUG */

void
Uart_Xfer(
    uint16 buflen,
    uint8p bufptr);

/* Changed the HBI deviceId data to match the MPI format S.H.
#define GET_CONNECTOR_ID(deviceId) (((deviceId) & 0x04)?1:0 )
#define GET_CHIP_SELECT(deviceId) ((deviceId) & 0x03)

#define GPI_WrData8(connector, cs, data, size) \
        GPI_Data(connector, cs, 0, GPI_WR_CMD, data, size)

*/
#define GET_CONNECTOR_ID(deviceId) (((deviceId) & 0x10)?0:1 )
/* #define GET_CHIP_SELECT(deviceId) (((deviceId) & 0x0C) >> 2)*/
/* 
 * This macro may look funny (because it is) but it had to be done to follow the
 * help pages. The help page inverts chip select 1 and 2 to follow the spi drivers
 * chip select access non-sense 
 */
#define GET_CHIP_SELECT(deviceId) ( (((deviceId) & 0x08) >> 3) | (((deviceId) & 0x04) >> 1))
#define GET_TRANSFER_SIZE(deviceId) ((deviceId) & 0x01)
#define WRITE_COMMAND 0
#define READ_COMMAND 1

#define MAX_BUF_LEN 1024
#define WORD_CHAR_CNVR_BUF_SIZE 512
#define JUNK_DATA 0xFF

static uint8 tempBuf[512]; /* All zero buffer */

static uint8p
CreateSpiRawXferBuf(
    VpDeviceIdType deviceId,
    uint8 numwords,
    uint16p data,
    uint8 ReadWrite,
    uint8p buffer,
    uint8 notBootWr,
    uint16 cmd);
    

/**
 * VpHalHbi(): Configures the HBI bus and glue logic (if any)
 *
 * This function performs four basic HBI calls. Command Word Read, Command Word
 * Write, Data Word Read and Data Word Write to/from the device specified by the
 * device id.
 *
 * Params:
 *  uint8 deviceId:     Device Id (chip select ID)
 *  bool isWordData:     indicates what the word pointer argument is used for
 *                        FALSE = Command Word, 
 *                        TRUE = Data Word.
 *  bool isWrite:       indicates if the word pointer is being read or written
 *                      to the device.
 *                        FALSE = Read into data buffer
 *                        TRUE  = Write from data buffer.
 *  uint16p word:       pointer to read/write data word.
 *  
 *
 * Returns:
 *  This function returns FALSE if some error occurred during HBI initialization
 *  or TRUE otherwise.
 */
/*****************************************************************************
 * HAL functions for VCP and VPP. Not necessary for CSLAC devices.
 ****************************************************************************/
    
bool 
VpHalHbi(
    VpDeviceIdType deviceId, 
    bool isWordData, 
    bool isWrite, 
    uint16p word)
{
    /* HBI MODE SUPPORT ONLY */
    if (! (deviceId & 0x80) ) {
        bool result = TRUE;

        if ((isWordData == FALSE) && (isWrite == TRUE)) {
            //printf("\t VpHalHbi Write Command Word 0x%04x GPI_WR_CMD= %i\n", *word, GPI_WR_CMD);
            /* Writing a Command Word */
            result = GPI_Command( GET_CONNECTOR_ID(deviceId), 
                                  GET_CHIP_SELECT(deviceId),
                                  GET_TRANSFER_SIZE(deviceId), 
                                  GPI_WR_CMD,
                                  *word,
                                  NULL, 
                                  0);
    
        } else if ((isWordData == FALSE) && (isWrite == FALSE)) {
            /* Reading a Command Word */
            //printf("VpHalHbi Read Command Word\n");
            result = GPI_Command( GET_CONNECTOR_ID(deviceId), 
                                  GET_CHIP_SELECT(deviceId),
                                  GET_TRANSFER_SIZE(deviceId), 
                                  (GPI_RD_CMD | GPI_RD_W_ADDR_HI), 
                                  0,
                                  (unsigned char *)word, 
                                  2);                              
                                
        } else if ((isWordData == TRUE) && (isWrite == TRUE)) {
            //printf("\t VpHalHbi Write Data Word 0x%04x GPI_WR_CMD= %i\n", *word, GPI_WR_CMD);
            /* Writing a Data Word */
            result = GPI_Data(  GET_CONNECTOR_ID(deviceId), 
                                GET_CHIP_SELECT(deviceId),
                                GET_TRANSFER_SIZE(deviceId), 
                                GPI_WR_CMD, 
                                (unsigned char *)word, 
                                2);
            
        } else if ((isWordData == TRUE) && (isWrite == FALSE)) {
            //printf("VpHalHbi Read Data Word\n");
            /* Reading a Data Word */
            result = GPI_Data(  GET_CONNECTOR_ID(deviceId), 
                                GET_CHIP_SELECT(deviceId),
                                GET_TRANSFER_SIZE(deviceId), 
                                GPI_RD_DATA, 
                                (unsigned char *)word, 
                                2);
        }
    } else {
        /* 
         * The deviceId is actually the cpld configuration between byte transfers
         * Ignore bit 5 and 6 because they are MPI specific 
         */
        uint8 cpldConfig = (deviceId & 0x9F);
        uint8 buflen = 0;
        uint8 buffer[6];
        uint16 cmd = *word;
        if (isWrite == FALSE) {
            cmd = 0xFFFF;
        }

        /* Only write cmd, write data and read data are supported. */
        if (!((isWordData == FALSE) && (isWrite == FALSE))) {
            /* Start building the data array */
            buffer[buflen++] = cpldConfig;
            buffer[buflen++] = (uint8)(cmd >> 8);
            
            /* add the rest of the data to the array based on transfer size */
            if (deviceId & 0x03){
                buffer[buflen++] = (uint8)cmd;
            } else {
                buffer[buflen++] = cpldConfig;
                buffer[buflen++] = (uint8)cmd;
            }
        } else {
            /* Spi mode does not support Read Command */
            return FALSE;
        }

        /* read or write the data */
        VpSysEnterCritical(deviceId, VP_MPI_CRITICAL_SEC);
        SPI_MPI_RAW_Xfer( buffer, buflen );
        VpSysExitCritical(deviceId, VP_MPI_CRITICAL_SEC);

        /* put the return data back into the word pointer */
        if (isWrite == FALSE) {        
            if (deviceId & 0x03) {   /* 16bit mode */
                word[0] = (((uint16)buffer[1] << 8) | (uint16)buffer[2]);
            } else {    /* 8bit mode */
                word[0] = (((uint16)buffer[1] << 8) | (uint16)buffer[3]);
            }
        }
    }

    return TRUE;
}   
    

/* SpiRawCmd dumps the input buffer onto the SPI
 * This function is called by SpiRawCommand() in hal.c
 *
 * Params:
 *  uint16* data the buffer to be sent over SPI
 *  uint16 length the length of the buffer
 *
 * Returns:
 *  This function returns the data pointer that has been sent
 */
uint8p SpiRawCmd(
    uint8p data,
    uint16 length)
{

    VpSysEnterCritical(data[0], VP_MPI_CRITICAL_SEC);

    SPI_MPI_RAW_Xfer( data, length );

    VpSysExitCritical(data[0], VP_MPI_CRITICAL_SEC);

    return data;
}

/**
 * VpHalHbiInit(): Configures the HBI bus and glue logic (if any)
 *
 * This function performs any tasks necessary to prepare the system for
 * communicating through the HBI, including writing the HBI configuration
 * register.  The HBI read and write functions should work after HbiHalInit()
 * is successfully executed. HbiHalInit() should be well-behaved even if called
 * more than once between system resets. HbiHalInit() is called from
 * VpBootLoad() since VpBootLoad() is normally the first VoicePath function
 * that the host application will call.
 *
 * This function is called by VpBootLoad() before sending the VCP firmware
 * image through the HBI.
 *
 * Params:
 *  uint8 deviceId: Device Id (chip select ID)
 *
 * Returns:
 *  This function returns FALSE if some error occurred during HBI initialization
 *  or TRUE otherwise.
 */
/*****************************************************************************
 * HAL functions for VCP and VPP. Not necessary for CSLAC devices.
 ****************************************************************************/
bool VpHalHbiInit(
    VpDeviceIdType deviceId)
{
    /*
     * Note that Setting up the basic device should be handled by the
     * some board bring up function. That function should setup the
     * CPLD based on the line module that is plugged in. Those functions
     * would configure the CPLD so that basic communication to the part
     * can happen between the HAL and the line module.
     */

    /* Write the HBI configuration register. */
    return VpHalHbiCmd(deviceId, HBI_CMD_CONFIGURE_INT + HBI_PINCONFIG);

} /* VpHalHbiInit() */

/**
 * VpHalHbiCmd(): Sends a command word over the HBI, with no data words.
 *
 *  Accepts a uint16 HBI command which is little-endian or big-endian,
 * depending on the host architecture.  Command words on the HBI bus are always
 * big-endian. This function is responsible for byte-swapping if required. This
 * implementation for Linux does not need any byte swapping.
 *
 * NOTE: VCP MUST use 16bit SPI or GPI because of a bug
 *
 * Params:
 * uint8 deviceId: Device Id (chip select ID)
 * uint16 cmd: the command word to send
 *
 * Returns:
 *   TRUE on success, FALSE on failure
 */
/*****************************************************************************
 * HAL functions for VCP and VPP. Not necessary for CSLAC devices.
 ****************************************************************************/
bool VpHalHbiCmd(
    VpDeviceIdType deviceId,
    uint16 cmd)
{

#ifdef HBI_HAL_DEBUG
    printf("Cmd %4.4X ", (int)cmd);
#endif /* HBI_HAL_DEBUG */

    /* GPI mode */
    /* for deviceId bit 7, 0 = GPI, 1 = SPI */
    if(!(deviceId & 0x80)){

        if(!GPI_Command(GET_CONNECTOR_ID(deviceId), GET_CHIP_SELECT(deviceId),
                GET_TRANSFER_SIZE(deviceId), WRITE_COMMAND, cmd, NULL, 0)) {
            goto VpHalHbiCmdSuccess;
        } else {
            goto VpHalHbiCmdFailure;
        }

    } else {
        /* SPI mode */
        uint8 cpldConfig;
        uint8 buflen = 0, buffer[MAX_BUF_LEN];
        uint8p bufptr = &buffer[0];

        /* The deviceId is actually the cpld configuration between byte transfers */
        /* Ignore bit 5 and 6 because they are MPI specific */
        cpldConfig = (deviceId & 0x9f);

        if ( (deviceId & 0x03) == 0x03 ){
            /* 16 bit mode */
            /* Start building the data array */
            buffer[buflen++] = cpldConfig;

            /*add 16bit command*/
            buffer[buflen++] = (uint8)(cmd>>8);
            buffer[buflen++] = (uint8)cmd;
        } else if ( (deviceId & 0x03) == 0x00) {
            /* 8 bit mode */
            /* Start building the data array */
            buffer[buflen++] = cpldConfig;
            /*add 16bit command*/
            buffer[buflen++] = (uint8)(cmd>>8);
            buffer[buflen++] = cpldConfig;
            buffer[buflen++] = (uint8)cmd;
        } else if ( (deviceId & 0x03) == 0x10) {
            /* 32bit mode */
            /* Start building the data array */
            buffer[buflen++] = cpldConfig;

            /*add 16bit command, add 16 bits of junk (0xFFFF) to the end */
            buffer[buflen++] = (uint8)(cmd>>8);
            buffer[buflen++] = (uint8)cmd;
            buffer[buflen++] = (uint8)JUNK_DATA;
            buffer[buflen++] = (uint8)JUNK_DATA;
        } else {
            /* 8 bit dummy mode */
            /* What goes here? */
        }

        VpSysEnterCritical(deviceId, VP_MPI_CRITICAL_SEC);

        /* deviceId bit 6 forces output to the serial port */
        /* disabled by (& 0x9f) - why was this added? -RS */
        if(deviceId & 0x40){
            Uart_Xfer(buflen, bufptr );
        }
        else{ /* send normally */
            SPI_MPI_RAW_Xfer( buffer, buflen );
        }

        VpSysExitCritical(deviceId, VP_MPI_CRITICAL_SEC);

        goto VpHalHbiCmdSuccess;
    }

VpHalHbiCmdSuccess:
#ifdef HBI_HAL_DEBUG
    printf("(ok)\n");
#endif /* HBI_HAL_DEBUG */
    return TRUE;

VpHalHbiCmdFailure:
#ifdef HBI_HAL_DEBUG
    printf("(fail)\n");
#endif /* HBI_HAL_DEBUG */
    return FALSE;

} /* VpHalHbiCmd() */

/**
 * VpHalHbiWrite(): Sends a command word and up to 256 data words over the HBI.
 *
 *  Accepts a uint16 HBI command which is little-endian or big-endian, depending
 * on the host architecture.  Command words on the HBI bus are always big-
 * endian.  This function is responsible for byte-swapping the command word, if
 * required. Note that this Linux implementation does not need byte swapping.
 *
 *  Accepts an array of uint16 data words.  No byte-swapping is necessary on
 * data words in this function.  Instead, the HBI bus can be configured in
 * VpHalHbiInit() to match the endianness of the host platform.
 *
 * Params:
 *   uint8 deviceId: Device Id (chip select ID)
 *   uint16 cmd: the command word to send
 *   uint8 numwords: the number of data words to send, minus 1
 *   uint16p data: the data itself; use data = (uint16p)0 to send
 *      zeroes for all data words
 *
 * Returns:
 *   TRUE on success, FALSE on failure
 */
/*****************************************************************************
 * HAL functions for VCP and VPP. Not necessary for CSLAC devices.
 ****************************************************************************/
bool VpHalHbiWrite(
    VpDeviceIdType deviceId,
    uint16 cmd,
    uint8 numwords,
    uint16p data)
{
    bool status;
    if (!VpHalHbiCmd(deviceId, cmd)) return FALSE;

#ifdef HBI_HAL_DEBUG
    {
        int i;
        printf("Write ");
        for (i = 0; i <= numwords; i++) {
            uint16 word;
            if (data == NULL) {
                word = 0;
            } else {
                word = data[i];
            }
            printf("%4.4X ", (int)word);
        }
    }
#endif /* HBI_HAL_DEBUG */

    status = VpHalHbiBootWr(deviceId, numwords, (VpImagePtrType)data);

#ifdef HBI_HAL_DEBUG
    if (status) {
        printf("(ok)\n");
    } else {
        printf("(fail)\n");
    }
#endif

    return status;
} /* VpHalHbiWrite() */

/**
 * VpHalHbiRead(): Sends a command, and receives up to 256 data words over the
 * HBI.
 *
 *  Accepts a uint16 HBI command which is little-endian or big-endian, depending
 * on the host architecture.  Command words on the HBI bus are always big-
 * endian.  This function is responsible for byte-swapping the command word, if
 * required.
 *
 * Retrieves an array of uint16 data words.  No byte-swapping is necessary on
 * data words in this function.  Instead, the HBI bus can be configured in
 * VpHalHbiInit() to match the endianness of the host platform.
 *
 * Params:
 *   uint8 deviceId: Device Id (chip select ID)
 *   uint8 numwords: the number of words to receive, minus 1
 *   uint16p data: where to put them
 *
 * Returns:
 *   TRUE on success, FALSE on failure
 */
/*****************************************************************************
 * HAL functions for VCP and VPP. Not necessary for CSLAC devices.
 ****************************************************************************/
bool VpHalHbiRead(
    VpDeviceIdType deviceId,
    uint16 cmd,
    uint8 numwords,
    uint16p data)
{
    bool retval;

#ifdef HBI_HAL_DEBUG
    int i;
    printf("Cmd %4.4X Read ", (int)cmd);
#endif /* HBI_HAL_DEBUG */

    /* Temporary spin-wait here because the VCP2 emulator can't handle
       full-speed HBI access. */
    {
        int i;
        for (i = 0; i < 10000; i++) {
            ;
        }
    }

    /* for deviceId bit 7, 0 = GPI, 1 = SPI */
    if(!(deviceId & 0x80)){
        int numBytes = (numwords + 1)* 2;

        if(!GPI_Command(GET_CONNECTOR_ID(deviceId), GET_CHIP_SELECT(deviceId),
                GET_TRANSFER_SIZE(deviceId), READ_COMMAND, cmd,
               (unsigned char *)data, numBytes)) {
            retval = TRUE;
            goto VpHalHbiReadReturn;
        } else {
            retval = FALSE;
            goto VpHalHbiReadReturn;
        }

    } else {
        /* SPI mode */
        uint8 Read = 0xFF, notBootWr = 1, buffer[MAX_BUF_LEN];
        uint16 buflen, i, rtnDataBufIdx = 4;
        uint8p bufptr = &buffer[0];

        numwords++;
        /* create buffer, 0x00 means write */
        CreateSpiRawXferBuf(deviceId, numwords, data, Read,
            bufptr, notBootWr, cmd);

        if(bufptr == NULL){
            retval = FALSE;
            goto VpHalHbiReadReturn;
        }

        /* 16 or 8 bit BootWr, add 3  (4) to include cmd */
        if(deviceId & 0x01){
            buflen = (numwords * 3) + 3;
        } else {
            buflen = (numwords * 4) + 4;
        }
        numwords--;
        VpSysEnterCritical(deviceId, VP_MPI_CRITICAL_SEC);
        /* 252 is largest valid value divisible by 3 and 4 */
        for(i = 0; i < buflen; i += 252) {
            uint8 tmplen;

            if( (buflen - i) > 252){
                tmplen = 252;
            } else {
                tmplen = buflen % 252;
            }

            /* deviceId bit 6 forces output to the serial port */
            if(deviceId & 0x40){
                Uart_Xfer(tmplen, bufptr );
            }
            else{ /* send normally */
            SPI_MPI_RAW_Xfer( &buffer[i], tmplen );

            }
        }

        VpSysExitCritical(deviceId, VP_MPI_CRITICAL_SEC);

        /* When you send a buffer for a read, you fill the buffer with 0xFF for each byte to be read.
        * Those 0xFFs are replaced by the correctly read data.  An example of 16bit reading 2 words from location 0:
        *
        * indx:   0    1    2    3    4    5    6    7    8    9    10   11
        * sent: 0x83 0x00 0x00 0x83 0x00 0x02 0x83 0xFF 0xFF 0x83 0xFF 0xFF
        * rtnd: 0x83 0x00 0x00 0x83 0x00 0x02 0x83 0xXX 0xXX 0x83 0xXX 0xXX
        *
        * where 0xXX is the returned data.
        * Thus, each transfer mode must parse the read data differently.
        */

        /* Placing result data back into users data pointer */

        if ( (deviceId & 0x03) == 0x03) {   /* 16bit mode */
            rtnDataBufIdx = 4;
            for (i = 0; rtnDataBufIdx <= buflen; rtnDataBufIdx+=3) {
                data[i++] = (((uint16)buffer[rtnDataBufIdx]<<8 & 0xFF00) | (uint16)buffer[rtnDataBufIdx+1]);
            }

        } else if ( (deviceId & 0x03) == 0x00) {    /* 8bit mode */
            rtnDataBufIdx = 5;
            for (i = 0; rtnDataBufIdx <= buflen; rtnDataBufIdx+=4) {
                data[i++] = (((uint16)buffer[rtnDataBufIdx]<<8 & 0xFF00) | (uint16)buffer[rtnDataBufIdx+2]);
            }

        } else if ( (deviceId & 0x03) == 0x02) {    /* 32bit mode */
            rtnDataBufIdx = 3;
            /* This is tricky because of the incrementing the loop variable inside the loop */
            for (i = 0; rtnDataBufIdx <= buflen; rtnDataBufIdx+=2) {
                data[i++] = (((uint16)buffer[rtnDataBufIdx]<<8 & 0xFF00) | (uint16)buffer[rtnDataBufIdx+1]);
                rtnDataBufIdx+=3;
                data[i++] = (((uint16)buffer[rtnDataBufIdx]<<8 & 0xFF00) | (uint16)buffer[rtnDataBufIdx+1]);
            }
        }

        retval = TRUE;
        goto VpHalHbiReadReturn;
    }

VpHalHbiReadReturn:
#ifdef HBI_HAL_DEBUG
    for (i = 0; i <= numwords; i++) {
        printf("%4.4X ", (int)data[i]);
    }

    if (retval) {
        printf("(ok)\n");
    } else {
        printf("(fail)\n");
    }
#endif /* HBI_HAL_DEBUG */

    return retval;
} /* VpHalHbiRead() */

/**
 * VpHalHbiBootWr():
 *
 *  This is used by the VpBootLoad() function to send the boot stream to the
 * VCP.  This function is separate from VpHalHbiWrite(), for the following
 * reasons:
 *
 *  1. This function does not accept a command word; only data words.
 *  2. This function accepts uint8 data, instead of uint16 data.  Be careful
 *     not to assume that this data is word-aligned in memory.
 *  3. The HBI must be configured for big-endian data words while the boot
 *     stream is being transmitted, regardless of the endianness of the host
 *     platform.  This is because the boot image is an opaque stream of HBI
 *     command words and data words.  Therefore, commands and data cannot be
 *     distinguished for separate treatment by this function.  Since HBI
 *     command words are always big-endian, data words have to be big-endian,
 *     too.  The boot stream is stored big-endian in memory, even on little-
 *     endian hosts.
 *        If VpHalHbiInit() configures the HBI for little-endian data words,
 *     then this function must temporarily change the configuration by calling
 *     VpHalHbiCmd(HBI_CMD_CONFIGURE(...)), and change it back before
 *     returning.  In such a case, this function will need to swap each pair
 *     of bytes in the boot stream before sending.
 *        Another possibility (as in the case of our test platform) is a
 *     little-endian host architecture, with the HBI bus configured for big-
 *     endian data words.  In this case, byte-swapping has to be done in
 *     VpHalHbiWrite() or in the glue logic between the host and the VCP.
 *     In these setups, VpHalHbiBootWr() does not need to reconfigure the
 *     HBI.
 *  4. This function takes a VpImagePtrType pointer to char data, which is a
 *     platform-dependent type defined in vp_hal.h.  In our system, a "_huge"
 *     pointer is needed for this function, since the boot stream is larger
 *     than 64 kB.
 *
 * Params
 *   uint8 deviceId: Device Id (chip select ID)
 *  'length' specifies the number of 16-bit words to write to the VCP.
 *  'pBuf' points into the boot stream.
 *
 * Returns
 *  HbiHalBootWr() returns TRUE on success, FALSE on failure.
 *
 * Notes
 *  THIS FUNCTION IS NOT REENTRANT!
 */
/*****************************************************************************
 * HAL functions for VCP and VPP. Not necessary for CSLAC devices.
 ****************************************************************************/
bool VpHalHbiBootWr(
    VpDeviceIdType deviceId,
    uint8 numwords,
    VpImagePtrType data)
{


/* data is a buffer of uint8 */
    if(!(deviceId & 0x80)){

        int numBytes = (numwords + 1)*2;

        if(!GPI_Data(GET_CONNECTOR_ID(deviceId), GET_CHIP_SELECT(deviceId),
                GET_TRANSFER_SIZE(deviceId), GPI_WR_CMD, data, numBytes)) {
            return TRUE;    /* success */
        } else {
            return FALSE;    /* Failure */
        }

    } else {  /* SPI mode */
        uint8 Write = 0, notBootWr = 0, buffer[MAX_BUF_LEN];
        uint16 buflen = 0, cmd = 0;
        uint8p bufptr = &buffer[0], bufend;
        uint16 maxChunkSize;

        /* Compute maximum amount of data that can be send to the CPLD. */
        if ((deviceId & 0x03) == 0x03) {
            maxChunkSize = 255;     /* 16-bit mode */
        } else {
            maxChunkSize = 254;     /* 8-bit mode */
        }

        numwords++;
        /* create buffer, 0x00 means write */
        bufend = CreateSpiRawXferBuf(deviceId, numwords, (uint16p)data, Write,
            bufptr, notBootWr, cmd);

        buflen = bufend - bufptr;
        if (buflen == 0) {
            return FALSE;
        }

        VpSysEnterCritical(deviceId, VP_MPI_CRITICAL_SEC);

        while (buflen > 0) {
            uint8 tmplen = maxChunkSize;

            if (tmplen > buflen) {
                tmplen = buflen;
            }

            if (deviceId & 0x40){
                /* Send data to serial port. */
                Uart_Xfer(tmplen, bufptr);
            } else {
                /* Send data normally. */

#ifdef HBI_HAL_DEBUG_LONG
{
    int c;
    printf("buflen = %d\n", (int)buflen);
    printf("SPIN_MPI_RAW_Xfer(");
    for (c = 0; c < tmplen; c++) {
        printf("%2.2X ", (int)bufptr[c]);
    }
    printf(")\n");
}
#endif
                SPI_MPI_RAW_Xfer( bufptr, tmplen );
            }
            bufptr += tmplen;
            buflen -= tmplen;
        }

        VpSysExitCritical(deviceId, VP_MPI_CRITICAL_SEC);


        return TRUE;
    }
} /* VpHalHbiBootWr() */

/*****************************************************************************
 * HAL functions for VPP. Not necessary for CSLAC and VCP devices.
 ****************************************************************************/
/**
 * VpHalHbiWrite8(): Sends a command word and up to 256 data words over the HBI.
 *
 *  Accepts a uint16 HBI command which is little-endian or big-endian, depending
 * on the host architecture.  Command words on the HBI bus are always big-
 * endian.  This function is responsible for byte-swapping the command word, if
 * required. Note that this Linux implementation does not need byte swapping.
 *
 *  Accepts an array of uint8 data words.  No byte-swapping is necessary on
 * data words in this function.  Instead, the HBI bus can be configured in
 * VpHalHbiInit() to match the endianness of the host platform.
 *
 * Params:
 *   uint8 deviceId: Device Id (chip select ID)
 *   uint16 cmd: the command word to send
 *   uint8 numwords: the number of data words to send, minus 1
 *   uint8p data: the data itself; use data = (uint8p)0 to send
 *   zeroes for all data words
 *
 * Returns:
 *   TRUE on success, FALSE on failure
 */
bool VpHalHbiWrite8(
    VpDeviceIdType deviceId,
    uint16 cmd,
    uint8 numwords,
    uint8p data)
{
   int numBytes = (numwords + 1)* 2;
   uint8 *srcPtr = data;

#ifdef HBI_HAL_DEBUG
    printf(" WR8 CMD <== %04hX\n", cmd);
#endif /* HBI_HAL_DEBUG */

    if(data == (uint8p)0) {
        /*
         * DevNotes: Not efficient; Kernal mode driver can be modified to handle
         * this
         */
        srcPtr = tempBuf;
    }

    if(!GPI_Command(GET_CONNECTOR_ID(deviceId), GET_CHIP_SELECT(deviceId),
            GET_TRANSFER_SIZE(deviceId), WRITE_COMMAND, cmd, srcPtr, numBytes)) {
        return TRUE;    /* success */
    } else {
        return FALSE;    /* Failure */
    }
} /* VpHalHbiWrite8() */

/*****************************************************************************
 * HAL functions for VPP. Not necessary for CSLAC and VCP devices.
 ****************************************************************************/
/**
 * VpHalHbiRead8(): Sends a command, and receives up to 256 data words over the
 * HBI.
 *
 *  Accepts a uint16 HBI command which is little-endian or big-endian, depending
 * on the host architecture.  Command words on the HBI bus are always big-
 * endian.  This function is responsible for byte-swapping the command word, if
 * required.
 *
 *  Retrieves an array of uint8 data bytes.  No byte-swapping is necessary on
 * data words in this function.  Instead, the HBI bus can be configured in
 * VpHalHbiInit() to match the endianness of the host platform.
 *
 * Params:
 *   uint8 deviceId: Device Id (chip select ID)
 *   uint8 numwords: the number of words to receive, minus 1
 *   uint8p data: where to put them
 *
 * Returns:
 *   TRUE on success, FALSE on failure
 */
bool VpHalHbiRead8(
    VpDeviceIdType deviceId,
    uint16 cmd,
    uint8 numwords,
    uint8p data)
{

    int numBytes = (numwords + 1)* 2;

#ifdef HBI_HAL_DEBUG
    printf(" RD8 CMD <== %04hX\n", cmd);
#endif /* HBI_HAL_DEBUG */

    if(!GPI_Command(GET_CONNECTOR_ID(deviceId), GET_CHIP_SELECT(deviceId),
            GET_TRANSFER_SIZE(deviceId), READ_COMMAND, cmd, data, numBytes)) {
        return TRUE;    /* success */
    } else {
        return FALSE;    /* Failure */
    }

} /* VpHalHbiRead8() */


/*****************************************************************************
 * HAL functions for CSLAC devices. Not necessary for VCP and VPP
 ****************************************************************************/
/**
 * VpMpiCmd()
 *  This function executes a Device MPI command through the MPI port. It
 * executes both read and write commands. The read or write operation is
 * determined by the "cmd" argument (odd = read, even = write). The caller must
 * ensure that the data array is large enough to hold the data being collected.
 * Because this command used hardware resources, this procedure is not
 * re-entrant.
 *
 * Preconditions:
 *  The device must be initialized.
 *
 * Postconditions:
 *   The data pointed to by dataPtr, using the command "cmd", with length
 * "cmdLen" has been sent to the MPI bus via the chip select associated with
 * deviceId.
 */
void
VpMpiCmd(
    VpDeviceIdType deviceId,    /**< Chip select, connector and 3 or 4 wire
                                 * interface for command
                                 */
    uint8 ecVal,        /**< Value to write to the EC register */
    uint8 cmd,          /**< Command number */
    uint8 cmdLen,       /**< Number of bytes used by command (cmd) */
    uint8 *dataPtr)     /**< Pointer to the data location */
{
#define EC_REG_Rd    0x4B

    int cpldConfig, threeOrFourWire, bitOverride = 0xFF;
    uint8 rtnDataBufIdx = 7, x = 0, i, buffer[255];

    /* The deviceId is actually the cpld configuration between byte transfers */
    cpldConfig = deviceId;

    /* The two most sig bits of the cpld register need to be set to a 1
     * (always). */
    cpldConfig |= 0xC0;

    /* check to see if the user has declared that the part has a 3 or 4 wire
     * interface. */
    threeOrFourWire = cpldConfig & 0x20;

    /* to do the writes the 0x20 bit in the cpld register has to be removed
     * this bit should only be enabled for 3 wire commands that are reading back
     * data. see for loop below */
    cpldConfig &= 0xDF;

    /* Start building the data array */
    buffer[x++] = cpldConfig;

    /* If a EC read is being preformed don't set the EC register */
    if (EC_REG_Rd == cmd) {
        rtnDataBufIdx = 3;
    } else {
        buffer[x++] = 0x4A;
        buffer[x++] = cpldConfig;
        buffer[x++] = ecVal;
        buffer[x++] = cpldConfig;
    }

    /* Determine if cmd is a read command, if so change the interface bit
       which currently reflects the 3 or 4 wire status of the part*/
    if (cmd & 0x01) {
        bitOverride = 0x00;
        cpldConfig |= threeOrFourWire;
    }

    /* Add command to buffer */
    buffer[x++] = cmd;

    /* If the command is a read fill the buffer with cpld register commands and
     * 0's otherwise fill it with the cpld register commands and the bytes
     * provided by the user */
    for (i = 0; i < cmdLen; i++) {
        buffer[x++] = cpldConfig;
        buffer[x++] = dataPtr[i] & bitOverride;
    }

    VpSysEnterCritical(deviceId, VP_MPI_CRITICAL_SEC);

    SPI_MPI_RAW_Xfer( buffer, x );

    VpSysExitCritical(deviceId, VP_MPI_CRITICAL_SEC);

    /* Placing result data back into users data pointer */
    if (cmd & 0x01) {
        for (i = 0; rtnDataBufIdx <= x; rtnDataBufIdx+=2) {
            dataPtr[i++] = buffer[rtnDataBufIdx];
        }
    }

    return;

} /* End VpMpiCmd */

/*****************************************************************************
 * HAL functions for CSLAC devices. Not necessary for VCP and VPP
 ****************************************************************************/

/*****************************************************************************
 * HAL function utilities for HBI. Not necessary for CSLAC devices.
 ****************************************************************************/

/**
 * UartXfer()
 *  This function transmits a buffer from VpHalHbiRead,
 *  VpHalHbiWrite and VpHalHbiBootWr over the serial port
 *
 * Preconditions:
 * a buffer of chars and the buffer length
 *
 * Postconditions:
 *  buffer sent over the serial port
 */
void
Uart_Xfer(
    uint16 buflen,
    uint8p bufptr)
{
    FILE* sPort;
    uint16  x = 0;

    printf("buflen: %d\n", buflen);
    /* open serial "out" port to display on terminal */
    sPort=fopen("/dev/ttyS0","w+");

    /* mirror data sent */
    printf("buffer sent over serial port: \n");
    for(x = 0; x < buflen; x++){
        printf("%02x ",bufptr[x]);
    }
    printf("\n");

    /* send data out RS232 */
    for(x = 0; x < buflen; x++){
        fprintf(sPort,"%02x",bufptr[x]);
    }

    return;
}

/*****************************************************************************
 * HAL function utilities for HBI. Not necessary for CSLAC devices.
 ****************************************************************************/

/**
 * CreateSpiRawXferBuf()
 *  This function creates the SPI_MPI_RAW_XFER buffer for VpHalHbiRead,
 *  VpHalHbiWrite and VpHalHbiBootWr
 *
 * Preconditions:
 * The calling funtion passes all necessary arguments
 *
 * Postconditions:
 *  a uint8 buffer of size MAX_BUF_LEN is passed to the calling funtion
 *  ready to be sent over SPI_MPI_RAW_XFER
 *
 * Returns:
 *  the address after the last byte written into the buffer.
 */
uint8p
CreateSpiRawXferBuf(
    VpDeviceIdType deviceId,
    uint8 numwords,
    uint16p data,
    uint8 ReadWrite,
    uint8p buffer,
    uint8 notBootWr,
    uint16 cmd)
{
    /* SPI mode */
    /* Can ONLY run VCP in 16bit SPI mode because of errata with 8bit mode */

    uint8 cpldConfig;
    uint16 x = 0, i;

    /* The deviceId is actually the cpld configuration between byte transfers */
    /* Ignore bit 5 and 6 because they are MPI specific */
    cpldConfig = (deviceId & 0x9f);

    /* Check for 16 bit mode */
    if ( (deviceId & 0x03) == 0x03) {

        /* Make sure buffer won't overflow */
        if((numwords * 3) > MAX_BUF_LEN){
            return buffer;
        }

        if(notBootWr){
            /* Start building the data array */
            buffer[x++] = cpldConfig;

            /*add 16bit command*/
            buffer[x++] = (uint8)(cmd>>8);
            buffer[x++] = (uint8)cmd;
        }
        /* If data = NULL fill numwords with zero */
        if (data == NULL) {
            for (i = 0; i < numwords; i++) {
                buffer[x++] = cpldConfig;
                buffer[x++] = 0x00;
                buffer[x++] = 0x00;
            }
        } else {
            /* Fill buffer with the cpld register commands and data
             * If it is a read, fill buffer with 0xFF */
            for (i = 0; i < numwords; i++) {
                buffer[x++] = cpldConfig;
                buffer[x++] = (uint8)((data[i]>>8) | ReadWrite);
                buffer[x++] = (uint8)(data[i] | ReadWrite);
            }
        }
    } else if ( (deviceId & 0x03) == 0x00) {    /* 8 bit mode */

        /* Make sure buffer won't overflow */
        if ((numwords * 4) > MAX_BUF_LEN) {
            return buffer;
        }

        if (notBootWr) {
            /* Start building the data array */
            buffer[x++] = cpldConfig;

            /*add 16bit command*/
            buffer[x++] = (uint8)(cmd>>8);
            buffer[x++] = cpldConfig;
            buffer[x++] = (uint8)cmd;
        }

        /* If data = NULL fill numwords with zero */
        if (data == NULL) {
            for (i = 0; i < numwords; i++) {
                buffer[x++] = cpldConfig;
                buffer[x++] = 0x00;
                buffer[x++] = cpldConfig;
                buffer[x++] = 0x00;
            }
        } else {
            /* Fill buffer with the cpld register commands and data
             * If it is a read, fill buffer with 0xFF */
            for (i = 0; i < numwords; i++) {
                buffer[x++] = cpldConfig;
                buffer[x++] = (uint8)((data[i]>>8) | ReadWrite);
                buffer[x++] = cpldConfig;
                buffer[x++] = (uint8)(data[i] | ReadWrite);
            }
        }
    } else if ( (deviceId & 0x03) == 0x02) {    /* 32 bit mode */
        /* Make sure buffer won't overflow */
        if ((numwords * 2.5) > MAX_BUF_LEN) {
            return buffer;
        }

        if (notBootWr) {
            /* Start building the data array */
            buffer[x++] = cpldConfig;

            /*add 16bit command*/
            buffer[x++] = (uint8)(cmd>>8);
            buffer[x++] = (uint8)cmd;
            if(data == NULL){
                buffer[x++] = 0x00;
                buffer[x++] = 0x00;
            } else {
                buffer[x++] = (uint8)((data[0]>>8) | ReadWrite);
                buffer[x++] = (uint8)(data[0] | ReadWrite);
            }
        }
        /* One word of data transmitted in first 32bit frame */

        /* If data = NULL fill numwords with zero */
        if (data == NULL) {
            for (i = 1; i < numwords; i+=2) {
                buffer[x++] = cpldConfig;
                buffer[x++] = 0x00;
                buffer[x++] = 0x00;
                buffer[x++] = 0x00;
                buffer[x++] = 0x00;
            }
        } else {
            /* Fill buffer with the cpld register commands and data
             * If it is a read, fill buffer with 0xFF */
            for (i = 1; i < numwords; i+=2) {
                buffer[x++] = cpldConfig;
                buffer[x++] = (uint8)((data[i]>>8) | ReadWrite);
                buffer[x++] = (uint8)(data[i] | ReadWrite);
                buffer[x++] = (uint8)((data[i+1]>>8) | ReadWrite);
                buffer[x++] = (uint8)(data[i+1] | ReadWrite);
            }
        }

        /* Check to see if we added an extra word of data */
        if( (x / 5) != numwords ) {
            buffer[--x] = JUNK_DATA;
            buffer[--x] = JUNK_DATA;

        }
    }

    #ifdef HBI_HAL_DEBUG_LONG

            printf("deviceid: 0x%02x, cpldconfig: 0x%02x\n", deviceId, cpldConfig);
            printf("cmd: 0x%04x, numwords: %u, buflen: %u\n", cmd, numwords, x);
            printf("readwrite: 0x%02x, notbootwr: %u\n", ReadWrite, notBootWr);
            for (i = 0; i < numwords; i++) {
                printf("0x%02x ",data[i]);
            }
            printf("\n");

            for (i = 0; i < x; i++) {
                printf("0x%02x ",buffer[i]);
            }
            printf("\n");

    #endif

    return &buffer[x];
}
